@@ -3,10 +3,11 @@ __license__ = 'GPL 3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, re
+import os, re, posixpath
 from itertools import cycle
 
 from calibre.customize.conversion import InputFormatPlugin, OptionRecommendation
+from polyglot.builtins import unicode_type, as_bytes
 
 ADOBE_OBFUSCATION =  'http://ns.adobe.com/pdf/enc#RC'
 IDPF_OBFUSCATION = 'http://www.idpf.org/2008/embedding'
@@ -17,7 +18,7 @@ def decrypt_font_data(key, data, algorithm):
     crypt_len = 1024 if is_adobe else 1040
     crypt = bytearray(data[:crypt_len])
     key = cycle(iter(bytearray(key)))
-    decrypt = bytes(bytearray(x^key.next() for x in crypt))
+    decrypt = bytes(bytearray(x^next(key) for x in crypt))
     return decrypt + data[crypt_len:]
 
 
@@ -32,10 +33,11 @@ class EPUBInput(InputFormatPlugin):
     name        = 'EPUB Input'
     author      = 'Kovid Goyal'
     description = 'Convert EPUB files (.epub) to HTML'
-    file_types  = set(['epub'])
+    file_types  = {'epub'}
     output_encoding = None
+    commit_name = 'epub_input'
 
-    recommendations = set([('page_breaks_before', '/', OptionRecommendation.MED)])
+    recommendations = {('page_breaks_before', '/', OptionRecommendation.MED)}
 
     def process_encryption(self, encfile, opf, log):
         from lxml import etree
@@ -53,8 +55,8 @@ class EPUBInput(InputFormatPlugin):
             if (scheme and scheme.lower() == 'uuid') or \
                     (item.text and item.text.startswith('urn:uuid:')):
                 try:
-                    key = bytes(item.text).rpartition(':')[-1]
-                    key = uuid.UUID(key).bytes
+                    key = item.text.rpartition(':')[-1]
+                    key = uuid.UUID(as_bytes(key)).bytes
                 except:
                     import traceback
                     traceback.print_exc()
@@ -80,14 +82,14 @@ class EPUBInput(InputFormatPlugin):
         return False
 
     def set_guide_type(self, opf, gtype, href=None, title=''):
-        # Set the titlepage guide entry
+        # Set the specified guide entry
         for elem in list(opf.iterguide()):
-            if elem.get('type', '').lower() == 'titlepage':
+            if elem.get('type', '').lower() == gtype:
                 elem.getparent().remove(elem)
 
         if href is not None:
             t = opf.create_guide_item(gtype, title, href)
-            for guide in opf.root.iterchildren('guide'):
+            for guide in opf.root.xpath('./*[local-name()="guide"]'):
                 guide.append(t)
                 return
             guide = opf.create_guide_element()
@@ -99,8 +101,25 @@ class EPUBInput(InputFormatPlugin):
         ''' If there is a reference to the cover/titlepage via manifest properties, convert to
         entries in the <guide> so that the rest of the pipeline picks it up. '''
         from calibre.ebooks.metadata.opf3 import items_with_property
-        removed = None
-        raster_cover_href = opf.epub3_raster_cover
+        removed = guide_titlepage_href = guide_titlepage_id = None
+
+        # Look for titlepages incorrectly marked in the <guide> as covers
+        guide_cover, guide_elem = None, None
+        for guide_elem in opf.iterguide():
+            if guide_elem.get('type', '').lower() == 'cover':
+                guide_cover = guide_elem.get('href', '').partition('#')[0]
+                break
+        if guide_cover:
+            spine = list(opf.iterspine())
+            if spine:
+                idref = spine[0].get('idref', '')
+                for x in opf.itermanifest():
+                    if x.get('id') == idref and x.get('href') == guide_cover:
+                        guide_titlepage_href = guide_cover
+                        guide_titlepage_id = idref
+                        break
+
+        raster_cover_href = opf.epub3_raster_cover or opf.raster_cover
         if raster_cover_href:
             self.set_guide_type(opf, 'cover', raster_cover_href, 'Cover Image')
         titlepage_id = titlepage_href = None
@@ -109,6 +128,8 @@ class EPUBInput(InputFormatPlugin):
             if href and tid:
                 titlepage_id, titlepage_href = tid, href.partition('#')[0]
                 break
+        if titlepage_href is None:
+            titlepage_href, titlepage_id = guide_titlepage_href, guide_titlepage_id
         if titlepage_href is not None:
             self.set_guide_type(opf, 'titlepage', titlepage_href, 'Title Page')
             spine = list(opf.iterspine())
@@ -128,6 +149,7 @@ class EPUBInput(InputFormatPlugin):
         means, at most one entry with type="cover" that points to a raster
         cover and at most one entry with type="titlepage" that points to an
         HTML titlepage. '''
+        from calibre.ebooks.oeb.base import OPF
         removed = None
         from lxml import etree
         guide_cover, guide_elem = None, None
@@ -136,6 +158,16 @@ class EPUBInput(InputFormatPlugin):
                 guide_cover = guide_elem.get('href', '').partition('#')[0]
                 break
         if not guide_cover:
+            raster_cover = opf.raster_cover
+            if raster_cover:
+                if guide_elem is None:
+                    g = opf.root.makeelement(OPF('guide'))
+                    opf.root.append(g)
+                else:
+                    g = guide_elem.getparent()
+                guide_cover = raster_cover
+                guide_elem = g.makeelement(OPF('reference'), attrib={'href':raster_cover, 'type':'cover'})
+                g.append(guide_elem)
             return
         spine = list(opf.iterspine())
         if not spine:
@@ -174,8 +206,6 @@ class EPUBInput(InputFormatPlugin):
         # entry will be used by the epub output plugin, the raster cover entry
         # by other output plugins.
 
-        from calibre.ebooks.oeb.base import OPF
-
         # Search for a raster cover identified in the OPF
         raster_cover = opf.raster_cover
 
@@ -207,7 +237,7 @@ class EPUBInput(InputFormatPlugin):
                 if k.endswith(attr):
                     return v
         try:
-            with open('META-INF/container.xml') as f:
+            with open('META-INF/container.xml', 'rb') as f:
                 root = etree.fromstring(f.read())
                 for r in root.xpath('//*[local-name()="rootfile"]'):
                     if attr(r, 'media-type') != "application/oebps-package+xml":
@@ -218,7 +248,7 @@ class EPUBInput(InputFormatPlugin):
                     path = os.path.join(os.getcwdu(), *path.split('/'))
                     if os.path.exists(path):
                         return path
-        except:
+        except Exception:
             import traceback
             traceback.print_exc()
 
@@ -259,19 +289,24 @@ class EPUBInput(InputFormatPlugin):
                 raise DRMError(os.path.basename(path))
         self.encrypted_fonts = self._encrypted_font_uris
 
-        epub3_nav = opf.epub3_nav
-        if epub3_nav is not None:
-            self.convert_epub3_nav(epub3_nav, opf, log)
-
         if len(parts) > 1 and parts[0]:
             delta = '/'.join(parts[:-1])+'/'
+
+            def normpath(x):
+                return posixpath.normpath(delta + elem.get('href'))
+
             for elem in opf.itermanifest():
-                elem.set('href', delta+elem.get('href'))
+                elem.set('href', normpath(elem.get('href')))
             for elem in opf.iterguide():
-                elem.set('href', delta+elem.get('href'))
+                elem.set('href', normpath(elem.get('href')))
 
         f = self.rationalize_cover3 if opf.package_version >= 3.0 else self.rationalize_cover2
         self.removed_cover = f(opf, log)
+        if self.removed_cover:
+            self.removed_items_to_ignore = (self.removed_cover,)
+        epub3_nav = opf.epub3_nav
+        if epub3_nav is not None:
+            self.convert_epub3_nav(epub3_nav, opf, log, options)
 
         for x in opf.itermanifest():
             if x.get('media-type', '') == 'application/x-dtbook+xml':
@@ -281,11 +316,22 @@ class EPUBInput(InputFormatPlugin):
         not_for_spine = set()
         for y in opf.itermanifest():
             id_ = y.get('id', None)
-            if id_ and y.get('media-type', None) in {
-                    'application/vnd.adobe-page-template+xml', 'application/vnd.adobe.page-template+xml',
-                    'application/adobe-page-template+xml', 'application/adobe.page-template+xml',
-                    'application/text'}:
-                not_for_spine.add(id_)
+            if id_:
+                mt = y.get('media-type', None)
+                if mt in {
+                        'application/vnd.adobe-page-template+xml',
+                        'application/vnd.adobe.page-template+xml',
+                        'application/adobe-page-template+xml',
+                        'application/adobe.page-template+xml',
+                        'application/text'
+                }:
+                    not_for_spine.add(id_)
+                ext = y.get('href', '').rpartition('.')[-1].lower()
+                if mt == 'text/plain' and ext in {'otf', 'ttf'}:
+                    # some epub authoring software sets font mime types to
+                    # text/plain
+                    not_for_spine.add(id_)
+                    y.set('media-type', 'application/font')
 
         seen = set()
         for x in list(opf.iterspine()):
@@ -303,11 +349,11 @@ class EPUBInput(InputFormatPlugin):
 
         return os.path.abspath(u'content.opf')
 
-    def convert_epub3_nav(self, nav_path, opf, log):
+    def convert_epub3_nav(self, nav_path, opf, log, opts):
         from lxml import etree
         from calibre.ebooks.chardet import xml_to_unicode
         from calibre.ebooks.oeb.polish.parsing import parse
-        from calibre.ebooks.oeb.base import EPUB_NS, XHTML, NCX_MIME, NCX
+        from calibre.ebooks.oeb.base import EPUB_NS, XHTML, NCX_MIME, NCX, urlnormalize, urlunquote, serialize
         from calibre.ebooks.oeb.polish.toc import first_child
         from tempfile import NamedTemporaryFile
         with lopen(nav_path, 'rb') as f:
@@ -322,7 +368,9 @@ class EPUBInput(InputFormatPlugin):
         def add_from_li(li, parent):
             href = text = None
             for x in li.iterchildren(XHTML('a'), XHTML('span')):
-                text = etree.tostring(x, method='text', encoding=unicode, with_tail=False).strip() or ' '.join(x.xpath('descendant-or-self::*/@title')).strip()
+                text = etree.tostring(
+                    x, method='text', encoding=unicode_type, with_tail=False).strip() or ' '.join(
+                            x.xpath('descendant-or-self::*/@title')).strip()
                 href = x.get('href')
                 if href:
                     if href.startswith('#'):
@@ -350,12 +398,30 @@ class EPUBInput(InputFormatPlugin):
                 if ol is not None:
                     process_nav_node(ol, navmap)
                     break
+        else:
+            return
 
         with NamedTemporaryFile(suffix='.ncx', dir=os.path.dirname(nav_path), delete=False) as f:
             f.write(etree.tostring(ncx, encoding='utf-8'))
-        ncx_id = opf.add_path_to_manifest(f.name, NCX_MIME)
+        ncx_href = os.path.relpath(f.name, os.getcwdu()).replace(os.sep, '/')
+        ncx_id = opf.create_manifest_item(ncx_href, NCX_MIME, append=True).get('id')
         for spine in opf.root.xpath('//*[local-name()="spine"]'):
             spine.set('toc', ncx_id)
+        opts.epub3_nav_href = urlnormalize(os.path.relpath(nav_path).replace(os.sep, '/'))
+        opts.epub3_nav_parsed = root
+        if getattr(self, 'removed_cover', None):
+            changed = False
+            base_path = os.path.dirname(nav_path)
+            for elem in root.xpath('//*[@href]'):
+                href, frag = elem.get('href').partition('#')[::2]
+                link_path = os.path.relpath(os.path.join(base_path, urlunquote(href)), base_path)
+                abs_href = urlnormalize(link_path)
+                if abs_href == self.removed_cover:
+                    changed = True
+                    elem.set('data-calibre-removed-titlepage', '1')
+            if changed:
+                with open(nav_path, 'wb') as f:
+                    f.write(serialize(root, 'application/xhtml+xml'))
 
     def postprocess_book(self, oeb, opts, log):
         rc = getattr(self, 'removed_cover', None)
@@ -368,5 +434,3 @@ class EPUBInput(InputFormatPlugin):
             spine = {x.href for x in oeb.spine}
             if (cover_toc_item is not None and cover_toc_item not in spine):
                 oeb.toc.item_that_refers_to_cover = cover_toc_item
-
-
