@@ -8,6 +8,7 @@ import os, re
 
 from calibre.customize.conversion import InputFormatPlugin, OptionRecommendation
 from calibre import guess_type
+from polyglot.builtins import iteritems, unicode_type
 
 FB2NS  = 'http://www.gribuser.ru/xml/fictionbook/2.0'
 FB21NS = 'http://www.gribuser.ru/xml/fictionbook/2.1'
@@ -21,19 +22,18 @@ class FB2Input(InputFormatPlugin):
     file_types  = {'fb2', 'fbz'}
     commit_name = 'fb2_input'
 
-    recommendations = set([
+    recommendations = {
         ('level1_toc', '//h:h1', OptionRecommendation.MED),
         ('level2_toc', '//h:h2', OptionRecommendation.MED),
         ('level3_toc', '//h:h3', OptionRecommendation.MED),
-        ])
+        }
 
-    options = set([
+    options = {
     OptionRecommendation(name='no_inline_fb2_toc',
         recommended_value=False, level=OptionRecommendation.LOW,
         help=_('Do not insert a Table of Contents at the beginning of the book.'
                 )
-        ),
-    ])
+        )}
 
     def convert(self, stream, options, file_ext, log,
                 accelerators):
@@ -71,11 +71,11 @@ class FB2Input(InputFormatPlugin):
         stylesheets = doc.xpath('//*[local-name() = "stylesheet" and @type="text/css"]')
         css = ''
         for s in stylesheets:
-            css += etree.tostring(s, encoding=unicode, method='text',
+            css += etree.tostring(s, encoding=unicode_type, method='text',
                     with_tail=False) + '\n\n'
         if css:
-            import cssutils, logging
-            parser = cssutils.CSSParser(fetcher=None,
+            import css_parser, logging
+            parser = css_parser.CSSParser(fetcher=None,
                     log=logging.getLogger('calibre.css'))
 
             XHTML_CSS_NAMESPACE = '@namespace "%s";\n' % XHTML_NS
@@ -83,7 +83,10 @@ class FB2Input(InputFormatPlugin):
             log.debug('Parsing stylesheet...')
             stylesheet = parser.parseString(text)
             stylesheet.namespaces['h'] = XHTML_NS
-            css = unicode(stylesheet.cssText).replace('h|style', 'h|span')
+            css = stylesheet.cssText
+            if isinstance(css, bytes):
+                css = css.decode('utf-8', 'replace')
+            css = css.replace('h|style', 'h|span')
             css = re.sub(r'name\s*=\s*', 'class=', css)
         self.extract_embedded_content(doc)
         log.debug('Converting XML to HTML...')
@@ -103,7 +106,7 @@ class FB2Input(InputFormatPlugin):
         notes = {a.get('href')[1:]: a for a in result.xpath('//a[@link_note and @href]') if a.get('href').startswith('#')}
         cites = {a.get('link_cite'): a for a in result.xpath('//a[@link_cite]') if not a.get('href', '')}
         all_ids = {x for x in result.xpath('//*/@id')}
-        for cite, a in cites.iteritems():
+        for cite, a in iteritems(cites):
             note = notes.get(cite, None)
             if note:
                 c = 1
