@@ -3,15 +3,15 @@ Retrieve and modify in-place Mobipocket book metadata.
 '''
 
 from __future__ import with_statement
+from __future__ import print_function
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal kovid@kovidgoyal.net and ' \
     'Marshall T. Vandegrift <llasram@gmail.com>'
 __docformat__ = 'restructuredtext en'
 
-import os
+import os, numbers, io
 from struct import pack, unpack
-from cStringIO import StringIO
 
 from calibre.ebooks import normalize
 from calibre.ebooks.mobi import MobiError, MAX_THUMB_DIMEN
@@ -20,12 +20,15 @@ from calibre.ebooks.mobi.langcodes import iana2mobi
 from calibre.utils.date import now as nowf
 from calibre.utils.imghdr import what
 from calibre.utils.localization import canonicalize_lang, lang_as_iso639_1
+from polyglot.builtins import unicode_type, range
+
 
 def is_image(ss):
     if ss is None:
         return False
     return what(None, ss[:200]) is not None
 
+
 class StreamSlicer(object):
 
     def __init__(self, stream, start=0, stop=None):
@@ -43,7 +46,7 @@ class StreamSlicer(object):
     def __getitem__(self, key):
         stream = self._stream
         base = self.start
-        if isinstance(key, (int, long)):
+        if isinstance(key, numbers.Integral):
             stream.seek(base + key)
             return stream.read(1)
         if isinstance(key, slice):
@@ -52,7 +55,7 @@ class StreamSlicer(object):
                 start, stop = stop, start
             size = stop - start
             if size <= 0:
-                return ""
+                return b""
             stream.seek(base + start)
             data = stream.read(size)
             if stride != 1:
@@ -63,7 +66,7 @@ class StreamSlicer(object):
     def __setitem__(self, key, value):
         stream = self._stream
         base = self.start
-        if isinstance(key, (int, long)):
+        if isinstance(key, numbers.Integral):
             if len(value) != 1:
                 raise ValueError("key and value lengths must match")
             stream.seek(base + key)
@@ -93,6 +96,7 @@ class StreamSlicer(object):
     def truncate(self, value):
         self._stream.truncate(value)
 
+
 class MetadataUpdater(object):
     DRM_KEY_SIZE = 48
 
@@ -101,7 +105,7 @@ class MetadataUpdater(object):
         data = self.data = StreamSlicer(stream)
         self.type = data[60:68]
 
-        if self.type != "BOOKMOBI":
+        if self.type != b"BOOKMOBI":
             return
 
         self.nrecs, = unpack('>H', data[76:78])
@@ -158,7 +162,7 @@ class MetadataUpdater(object):
         nitems, = unpack('>I', exth[8:12])
         pos = 12
         # Store any EXTH fields not specifiable in GUI
-        for i in xrange(nitems):
+        for i in range(nitems):
             id, size = unpack('>II', exth[pos:pos + 8])
             content = exth[pos + 8: pos + size]
             pos += size
@@ -219,7 +223,7 @@ class MetadataUpdater(object):
 
     def create_exth(self, new_title=None, exth=None):
         # Add an EXTH block to record 0, rewrite the stream
-        if isinstance(new_title, unicode):
+        if isinstance(new_title, unicode_type):
             new_title = new_title.encode(self.codec, 'replace')
 
         # Fetch the existing title
@@ -240,9 +244,9 @@ class MetadataUpdater(object):
 
         if not exth:
             # Construct an empty EXTH block
-            pad = '\0' * 4
-            exth = ['EXTH', pack('>II', 12, 0), pad]
-            exth = ''.join(exth)
+            pad = b'\0' * 4
+            exth = [b'EXTH', pack('>II', 12, 0), pad]
+            exth = b''.join(exth)
 
         # Update drm_offset(0xa8), title_offset(0x54)
         if self.encryption_type != 0:
@@ -256,7 +260,7 @@ class MetadataUpdater(object):
             self.record0[0x58:0x5c] = pack('>L', len(new_title))
 
         # Create an updated Record0
-        new_record0 = StringIO()
+        new_record0 = io.BytesIO()
         new_record0.write(self.record0[:0x10 + mobi_header_length])
         new_record0.write(exth)
         if self.encryption_type != 0:
@@ -265,9 +269,9 @@ class MetadataUpdater(object):
 
         # Pad to a 4-byte boundary
         trail = len(new_record0.getvalue()) % 4
-        pad = '\0' * (4 - trail)  # Always pad w/ at least 1 byte
+        pad = b'\0' * (4 - trail)  # Always pad w/ at least 1 byte
         new_record0.write(pad)
-        new_record0.write('\0'*(1024*8))
+        new_record0.write(b'\0'*(1024*8))
 
         # Rebuild the stream, update the pdbrecords pointers
         self.patchSection(0,new_record0.getvalue())
@@ -286,11 +290,11 @@ class MetadataUpdater(object):
             s = s.translate(FILTER)
             result += "%04X   %-*s   %s\n" % (N, length*3, hexa, s)
             N+=length
-        print result
+        print(result)
 
     def get_pdbrecords(self):
         pdbrecords = []
-        for i in xrange(self.nrecs):
+        for i in range(self.nrecs):
             offset, a1,a2,a3,a4 = unpack('>LBBBB', self.data[78+i*8:78+i*8+8])
             flags, val = a1, a2<<16|a3<<8|a4
             pdbrecords.append([offset, flags, val])
@@ -305,11 +309,11 @@ class MetadataUpdater(object):
 
     def dump_pdbrecords(self):
         # Diagnostic
-        print "MetadataUpdater.dump_pdbrecords()"
-        print "%10s %10s %10s" % ("offset","flags","val")
-        for i in xrange(len(self.pdbrecords)):
+        print("MetadataUpdater.dump_pdbrecords()")
+        print("%10s %10s %10s" % ("offset","flags","val"))
+        for i in range(len(self.pdbrecords)):
             pdbrecord = self.pdbrecords[i]
-            print "%10X %10X %10X" % (pdbrecord[0], pdbrecord[1], pdbrecord[2])
+            print("%10X %10X %10X" % (pdbrecord[0], pdbrecord[1], pdbrecord[2]))
 
     def record(self, n):
         if n >= self.nrecs:
@@ -323,14 +327,15 @@ class MetadataUpdater(object):
 
     def update(self, mi):
         mi.title = normalize(mi.title)
+
         def update_exth_record(rec):
             recs.append(rec)
             if rec[0] in self.original_exth_records:
                 self.original_exth_records.pop(rec[0])
 
-        if self.type != "BOOKMOBI":
-                raise MobiError("Setting metadata only supported for MOBI files of type 'BOOK'.\n"
-                                "\tThis is a %r file of type %r" % (self.type[0:4], self.type[4:8]))
+        if self.type != b"BOOKMOBI":
+            raise MobiError("Setting metadata only supported for MOBI files of type 'BOOK'.\n"
+                            "\tThis is a %r file of type %r" % (self.type[0:4], self.type[4:8]))
 
         recs = []
         added_501 = False
@@ -376,9 +381,9 @@ class MetadataUpdater(object):
                 update_exth_record((501, b'PDOC'))
 
         if mi.pubdate:
-            update_exth_record((106, str(mi.pubdate).encode(self.codec, 'replace')))
+            update_exth_record((106, unicode_type(mi.pubdate).encode(self.codec, 'replace')))
         elif mi.timestamp:
-            update_exth_record((106, str(mi.timestamp).encode(self.codec, 'replace')))
+            update_exth_record((106, unicode_type(mi.timestamp).encode(self.codec, 'replace')))
         elif self.timestamp:
             update_exth_record((106, self.timestamp))
         else:
@@ -417,15 +422,15 @@ class MetadataUpdater(object):
             recs.append((id, self.original_exth_records[id]))
         recs = sorted(recs, key=lambda x:(x[0],x[0]))
 
-        exth = StringIO()
+        exth = io.BytesIO()
         for code, data in recs:
             exth.write(pack('>II', code, len(data) + 8))
             exth.write(data)
         exth = exth.getvalue()
         trail = len(exth) % 4
-        pad = '\0' * (4 - trail)  # Always pad w/ at least 1 byte
-        exth = ['EXTH', pack('>II', len(exth) + 12, len(recs)), exth, pad]
-        exth = ''.join(exth)
+        pad = b'\0' * (4 - trail)  # Always pad w/ at least 1 byte
+        exth = [b'EXTH', pack('>II', len(exth) + 12, len(recs)), exth, pad]
+        exth = b''.join(exth)
 
         if getattr(self, 'exth', None) is None:
             raise MobiError('No existing EXTH record. Cannot update metadata.')
@@ -457,11 +462,13 @@ class MetadataUpdater(object):
                         self.thumbnail_record[:] = thumbnail
                 return
 
+
 def set_metadata(stream, mi):
     mu = MetadataUpdater(stream)
     mu.update(mi)
     return
 
+
 def get_metadata(stream):
     from calibre.ebooks.metadata import MetaInformation
     from calibre.ptempfile import TemporaryDirectory
@@ -473,8 +480,8 @@ def get_metadata(stream):
     stream.seek(0)
     try:
         raw = stream.read(3)
-    except:
-        raw = ''
+    except Exception:
+        raw = b''
     stream.seek(0)
     if raw == b'TPZ':
         from calibre.ebooks.metadata.topaz import get_metadata
@@ -513,8 +520,8 @@ def get_metadata(stream):
     else:
         try:
             data  = mh.section_data(mh.first_image_index)
-        except:
-            data = ''
+        except Exception:
+            data = b''
     if data and what(None, data) in {'jpg', 'jpeg', 'gif', 'png', 'bmp', 'webp'}:
         try:
             mi.cover_data = ('jpg', save_cover_data_to(data))
