@@ -9,7 +9,10 @@ __docformat__ = 'restructuredtext en'
 
 import time
 from functools import partial
-from Queue import Queue, Empty
+try:
+    from queue import Empty, Queue
+except ImportError:
+    from Queue import Empty, Queue
 
 
 from calibre.ebooks.metadata import check_isbn
@@ -20,9 +23,11 @@ from calibre import as_unicode
 NAMESPACES = {
               'openSearch':'http://a9.com/-/spec/opensearchrss/1.0/',
               'atom' : 'http://www.w3.org/2005/Atom',
-              'db': 'http://www.douban.com/xmlns/',
+              'db': 'https://www.douban.com/xmlns/',
               'gd': 'http://schemas.google.com/g/2005'
             }
+
+
 def get_details(browser, url, timeout):  # {{{
     try:
         if Douban.DOUBAN_API_KEY and Douban.DOUBAN_API_KEY != '':
@@ -39,6 +44,7 @@ def get_details(browser, url, timeout):  # {{{
     return raw
 # }}}
 
+
 def to_metadata(browser, log, entry_, timeout):  # {{{
     from lxml import etree
     from calibre.ebooks.chardet import xml_to_unicode
@@ -69,7 +75,7 @@ def to_metadata(browser, log, entry_, timeout):  # {{{
             log.exception('Programming error:')
         return None
 
-    id_url = entry_id(entry_)[0].text
+    id_url = entry_id(entry_)[0].text.replace('http://', 'https://')
     douban_id = id_url.split('/')[-1]
     title_ = ': '.join([x.text for x in title(entry_)]).strip()
     authors = [x.text.strip() for x in creator(entry_) if x.text]
@@ -144,14 +150,16 @@ def to_metadata(browser, log, entry_, timeout):  # {{{
     return mi
 # }}}
 
+
 class Douban(Source):
 
     name = 'Douban Books'
     author = 'Li Fanxi'
-    version = (2, 0, 0)
+    version = (2, 1, 0)
+    minimum_calibre_version = (2, 80, 0)
 
     description = _('Downloads metadata and covers from Douban.com. '
-            'Useful only for chinese language books.')
+            'Useful only for Chinese language books.')
 
     capabilities = frozenset(['identify', 'cover'])
     touched_fields = frozenset(['title', 'authors', 'tags',
@@ -161,7 +169,7 @@ class Douban(Source):
     cached_cover_url_is_reliable = True
 
     DOUBAN_API_KEY = '0bd1672394eb1ebf2374356abec15c3d'
-    DOUBAN_BOOK_URL = 'http://book.douban.com/subject/%s/'
+    DOUBAN_BOOK_URL = 'https://book.douban.com/subject/%s/'
 
     def get_book_url(self, identifiers):  # {{{
         db = identifiers.get('douban', None)
@@ -170,10 +178,13 @@ class Douban(Source):
     # }}}
 
     def create_query(self, log, title=None, authors=None, identifiers={}):  # {{{
-        from urllib import urlencode
-        SEARCH_URL = 'http://api.douban.com/book/subjects?'
-        ISBN_URL = 'http://api.douban.com/book/subject/isbn/'
-        SUBJECT_URL = 'http://api.douban.com/book/subject/'
+        try:
+            from urllib.parse import urlencode
+        except ImportError:
+            from urllib import urlencode
+        SEARCH_URL = 'https://api.douban.com/book/subjects?'
+        ISBN_URL = 'https://api.douban.com/book/subject/isbn/'
+        SUBJECT_URL = 'https://api.douban.com/book/subject/'
 
         q = ''
         t = None
@@ -191,14 +202,14 @@ class Douban(Source):
             title_tokens = list(self.get_title_tokens(title))
             if title_tokens:
                 q += build_term('title', title_tokens)
-            author_tokens = self.get_author_tokens(authors,
-                    only_first_author=True)
+            author_tokens = list(self.get_author_tokens(authors,
+                    only_first_author=True))
             if author_tokens:
                 q += ((' ' if q != '' else '') +
                     build_term('author', author_tokens))
             t = 'search'
         q = q.strip()
-        if isinstance(q, unicode):
+        if isinstance(q, type(u'')):
             q = q.encode('utf-8')
         if not q:
             return None
@@ -335,6 +346,7 @@ class Douban(Source):
         return None
     # }}}
 
+
 if __name__ == '__main__':  # tests {{{
     # To run these test use: calibre-debug -e src/calibre/ebooks/metadata/sources/douban.py
     from calibre.ebooks.metadata.sources.test import (test_identify_plugin,
@@ -356,4 +368,3 @@ if __name__ == '__main__':  # tests {{{
             ),
     ])
 # }}}
-
