@@ -12,7 +12,10 @@ __docformat__ = 'restructuredtext en'
 # bitcoins to 1E6CRSLY1uNstcZjLYZBHRVs1CPKbdi4ep
 
 import re
-from Queue import Queue, Empty
+try:
+    from queue import Empty, Queue
+except ImportError:
+    from Queue import Empty, Queue
 
 from calibre import as_unicode, replace_entities
 from calibre.ebooks.metadata import check_isbn
@@ -22,6 +25,8 @@ from calibre.ebooks.metadata.book.base import Metadata
 
 class Ozon(Source):
     name = 'OZON.ru'
+    minimum_calibre_version = (2, 80, 0)
+    version = (1, 1, 0)
     description = _('Downloads metadata and covers from OZON.ru (updated)')
 
     capabilities = frozenset(['identify', 'cover'])
@@ -35,7 +40,7 @@ class Ozon(Source):
     supports_gzip_transfer_encoding = True
     has_html_comments = True
 
-    ozon_url = 'http://www.ozon.ru'
+    ozon_url = 'https://www.ozon.ru'
 
     # match any ISBN10/13. From "Regular Expressions Cookbook"
     isbnPattern = r'(?:ISBN(?:-1[03])?:? )?(?=[-0-9 ]{17}|' \
@@ -52,12 +57,15 @@ class Ozon(Source):
     )
 
     def get_book_url(self, identifiers):  # {{{
-        import urllib2
+        try:
+            from urllib.parse import quote
+        except ImportError:
+            from urllib import quote
         ozon_id = identifiers.get('ozon', None)
         res = None
         if ozon_id:
             # no affiliateId is used in search/detail
-            url = '{}/context/detail/id/{}'.format(self.ozon_url, urllib2.quote(ozon_id), _get_affiliateId())
+            url = '{}/context/detail/id/{}'.format(self.ozon_url, quote(ozon_id), _get_affiliateId())
             res = ('ozon', ozon_id, url)
         return res
 
@@ -79,30 +87,32 @@ class Ozon(Source):
 
         ozonid = identifiers.get('ozon', None)
 
-        qItems = set([ozonid, isbn])
+        qItems = {ozonid, isbn}
 
         # Added Russian variant of 'Unknown'
         unk = [_('Unknown').upper(), 'Неизв.'.upper(), icu_upper('Неизв.')]
 
-        if title and title not in unk:
-            qItems.add(title)
+        # use only ozonid if specified otherwise ozon.ru does not like a combination
+        if not ozonid:
+            if title and title not in unk:
+                qItems.add(title)
 
-        if authors:
-            for auth in authors:
-                if icu_upper(auth) not in unk:
-                    qItems.add(auth)
+            if authors:
+                for auth in authors:
+                    if icu_upper(auth) not in unk:
+                        qItems.add(auth)
 
         qItems.discard(None)
         qItems.discard('')
         searchText = u' '.join(qItems).strip()
 
-        if isinstance(searchText, unicode):
+        if isinstance(searchText, type(u'')):
             searchText = searchText.encode('utf-8')
         if not searchText:
             return None
 
         search_url += quote_plus(searchText)
-        log.debug(u'search url: %r' % search_url)
+        log.debug(u'search url: %s' % search_url)
         return search_url
 
     # }}}
@@ -112,6 +122,7 @@ class Ozon(Source):
         from calibre.ebooks.chardet import xml_to_unicode
         from HTMLParser import HTMLParser
         from lxml import etree, html
+        import json
 
         if not self.is_configured():
             return
@@ -131,8 +142,11 @@ class Ozon(Source):
             doc = html.fromstring(xml_to_unicode(raw, verbose=True)[0])
             entries_block = doc.xpath(u'//div[@class="bSearchResult"]')
 
+            # log.debug(u'HTML: %s' % xml_to_unicode(raw, verbose=True)[0])
+
             if entries_block:
                 entries = doc.xpath(u'//div[contains(@itemprop, "itemListElement")]')
+                # log.debug(u'entries_block')
                 # for entry in entries:
                 #   log.debug('entries %s' % entree.tostring(entry))
                 metadata = self.get_metadata(log, entries, title, authors, identifiers)
@@ -140,19 +154,34 @@ class Ozon(Source):
             else:
                 # Redirect page: trying to extract ozon_id from javascript data
                 h = HTMLParser()
-                entry_string = (h.unescape(unicode(etree.tostring(doc, pretty_print=True))))
-                id_title_pat = re.compile(u'products":\[{"id":(\d{7}),"name":"([а-яА-Я :\-0-9]+)')
-                # result containing ozon_id and entry_title
-                entry_info = re.search(id_title_pat, entry_string)
-                ozon_id = entry_info.group(1) if entry_info else None
-                entry_title = entry_info.group(2) if entry_info else None
+                entry_string = (h.unescape(etree.tostring(doc, pretty_print=True, encoding='unicode')))
+                json_pat = re.compile(r'dataLayer\s*=\s*(.+)?;')
+                json_info = re.search(json_pat, entry_string)
+                jsondata = json_info.group(1) if json_info else None
+                if jsondata:
+                    idx = jsondata.rfind('}]')
+                    if idx > 0:
+                        jsondata = jsondata[:idx + 2]
 
-                if ozon_id:
-                    metadata = self.to_metadata_for_single_entry(log, ozon_id, entry_title, authors)
-                    identifiers['ozon'] = ozon_id
-                    self.get_all_details(log, [metadata], abort, result_queue, identifiers, timeout, cachedPagesDict={})
-                else:
-                    log.error('No SearchResults in Ozon.ru response found')
+                # log.debug(u'jsondata: %s' % jsondata)
+                dataLayer = json.loads(jsondata) if jsondata else None
+
+                ozon_id = None
+                if dataLayer and dataLayer[0] and 'ecommerce' in dataLayer[0]:
+                    jsproduct = dataLayer[0]['ecommerce']['detail']['products'][0]
+                    ozon_id = as_unicode(jsproduct['id'])
+                    entry_title = as_unicode(jsproduct['name'])
+
+                    log.debug(u'ozon_id %s' % ozon_id)
+                    log.debug(u'entry_title %s' % entry_title)
+
+                    if ozon_id:
+                        metadata = self.to_metadata_for_single_entry(log, ozon_id, entry_title, authors)
+                        identifiers['ozon'] = ozon_id
+                        self.get_all_details(log, [metadata], abort, result_queue, identifiers, timeout, cachedPagesDict={})
+
+                if not ozon_id:
+                    log.error('No SearchResults in Ozon.ru response found!')
 
         except Exception as e:
             log.exception('Failed to parse identify results')
@@ -175,16 +204,16 @@ class Ozon(Source):
 
         reRemoveFromTitle = re.compile(r'[?!:.,;+-/&%"\'=]')
 
-        title = unicode(title).upper() if title else ''
+        title = type(u'')(title).upper() if title else ''
         if reRemoveFromTitle:
             title = reRemoveFromTitle.sub('', title)
         authors = map(_normalizeAuthorNameWithInitials,
-                      map(unicode.upper, map(unicode, authors))) if authors else None
+                      map(type(u'').upper, map(type(u''), authors))) if authors else None
 
         ozon_id = identifiers.get('ozon', None)
         # log.debug(u'ozonid: ', ozon_id)
 
-        unk = unicode(_('Unknown')).upper()
+        unk = type(u'')(_('Unknown')).upper()
 
         if title == unk:
             title = None
@@ -203,7 +232,7 @@ class Ozon(Source):
         def calc_source_relevance(mi):  # {{{
             relevance = 0
             if title:
-                mititle = unicode(mi.title).upper() if mi.title else ''
+                mititle = type(u'')(mi.title).upper() if mi.title else ''
 
                 if reRemoveFromTitle:
                     mititle = reRemoveFromTitle.sub('', mititle)
@@ -217,7 +246,7 @@ class Ozon(Source):
                 relevance += 1
 
             if authors:
-                miauthors = map(unicode.upper, map(unicode, mi.authors)) if mi.authors else []
+                miauthors = map(type(u'').upper, map(type(u''), mi.authors)) if mi.authors else []
                 # log.debug('Authors %s vs miauthors %s'%(','.join(authors), ','.join(miauthors)))
 
                 if (in_authors(authors, miauthors)):
@@ -297,13 +326,13 @@ class Ozon(Source):
     # }}}
 
     def to_metadata(self, log, entry):  # {{{
-        title = unicode(entry.xpath(u'normalize-space(.//div[@itemprop="name"][1]/text())'))
+        title = type(u'')(entry.xpath(u'normalize-space(.//div[@itemprop="name"][1]/text())'))
         # log.debug(u'Title: -----> %s' % title)
 
-        author = unicode(entry.xpath(u'normalize-space(.//div[contains(@class, "mPerson")])'))
+        author = type(u'')(entry.xpath(u'normalize-space(.//div[contains(@class, "mPerson")])'))
         # log.debug(u'Author: -----> %s' % author)
 
-        norm_authors = map(_normalizeAuthorNameWithInitials, map(unicode.strip, unicode(author).split(u',')))
+        norm_authors = map(_normalizeAuthorNameWithInitials, map(type(u'').strip, type(u'')(author).split(u',')))
         mi = Metadata(title, norm_authors)
 
         ozon_id = entry.get('data-href').split('/')[-2]
@@ -321,7 +350,7 @@ class Ozon(Source):
 
         pub_year = None
         pub_year_block = entry.xpath(u'.//div[@class="bOneTileProperty"]/text()')
-        year_pattern = re.compile('\d{4}')
+        year_pattern = re.compile(r'\d{4}')
         if pub_year_block:
             pub_year = re.search(year_pattern, pub_year_block[0])
             if pub_year:
@@ -478,7 +507,7 @@ class Ozon(Source):
         langs_elem = doc.xpath(u'//div[contains(text(), "зык")]')
         if langs_elem:
             langs_elem = langs_elem[0].getnext()
-            langs = langs_elem.xpath(u'text()')[0].strip()
+            langs = langs_elem.xpath(u'text()')[0].strip() if langs_elem else None
         if langs:
             lng_splt = langs.split(u',')
             if lng_splt:
@@ -501,7 +530,7 @@ class Ozon(Source):
         # comments, from Javascript data
         beginning = fullString.find(u'FirstBlock')
         end = fullString.find(u'}', beginning)
-        comments = unicode(fullString[beginning + 75:end - 1]).decode("unicode-escape")
+        comments = type(u'')(fullString[beginning + 75:end - 1]).decode("unicode-escape")
         metadata.comments = replace_entities(comments, 'utf-8')
         # }}}
 
@@ -522,7 +551,7 @@ def _translateToBigCoverUrl(coverUrl):  # {{{
     # http://www.ozon.ru/multimedia/books_covers/1009493080.jpg
     m = re.match(r'.+\/([^\.\\]+).+$', coverUrl)
     if m:
-        coverUrl = 'http://www.ozon.ru/multimedia/books_covers/' + m.group(1) + '.jpg'
+        coverUrl = 'https://www.ozon.ru/multimedia/books_covers/' + m.group(1) + '.jpg'
     return coverUrl
 
 
@@ -576,11 +605,11 @@ def _format_isbn(log, isbn):  # {{{
             log.error('cannot format ISBN %s. Fow now only russian ISBNs are supported' % isbn)
     return res
 
-
 # }}}
 
+
 def _translageLanguageToCode(displayLang):  # {{{
-    displayLang = unicode(displayLang).strip() if displayLang else None
+    displayLang = type(u'')(displayLang).strip() if displayLang else None
     langTbl = {None: 'ru',
                u'Русский': 'ru',
                u'Немецкий': 'de',
@@ -602,11 +631,11 @@ def _translageLanguageToCode(displayLang):  # {{{
 def _normalizeAuthorNameWithInitials(name):  # {{{
     res = name
     if name:
-        re1 = u'^(?P<lname>\S+)\s+(?P<fname>[^\d\W]\.)(?:\s*(?P<mname>[^\d\W]\.))?$'
-        re2 = u'^(?P<fname>[^\d\W]\.)(?:\s*(?P<mname>[^\d\W]\.))?\s+(?P<lname>\S+)$'
-        matcher = re.match(re1, unicode(name), re.UNICODE)
+        re1 = r'^(?P<lname>\S+)\s+(?P<fname>[^\d\W]\.)(?:\s*(?P<mname>[^\d\W]\.))?$'
+        re2 = r'^(?P<fname>[^\d\W]\.)(?:\s*(?P<mname>[^\d\W]\.))?\s+(?P<lname>\S+)$'
+        matcher = re.match(re1, type(u'')(name), re.UNICODE)
         if not matcher:
-            matcher = re.match(re2, unicode(name), re.UNICODE)
+            matcher = re.match(re2, type(u'')(name), re.UNICODE)
 
         if matcher:
             d = matcher.groupdict()
@@ -630,7 +659,7 @@ def toPubdate(log, yearAsString):  # {{{
 # }}}
 
 def _listToUnicodePrintStr(lst):  # {{{
-    return u'[' + u', '.join(unicode(x) for x in lst) + u']'
+    return u'[' + u', '.join(type(u'')(x) for x in lst) + u']'
 
 
 # }}}
