@@ -9,7 +9,7 @@ __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 import re
 
 from lxml.etree import XMLParser, fromstring, XMLSyntaxError
-import cssutils
+import css_parser
 
 from calibre import force_unicode, human_readable, prepare_string_for_xml
 from calibre.ebooks.chardet import replace_encoding_declarations, find_declared_encoding
@@ -18,6 +18,7 @@ from calibre.ebooks.oeb.polish.pretty import pretty_script_or_style as fix_style
 from calibre.ebooks.oeb.polish.utils import PositionFinder, guess_type
 from calibre.ebooks.oeb.polish.check.base import BaseError, WARN, ERROR, INFO
 from calibre.ebooks.oeb.base import OEB_DOCS, XHTML_NS, urlquote, URL_SAFE, XHTML
+from polyglot.builtins import iteritems, unicode_type, error_message
 
 HTML_ENTITTIES = frozenset(html5_entities)
 XML_ENTITIES = {'lt', 'gt', 'amp', 'apos', 'quot'}
@@ -102,7 +103,7 @@ class NamedEntities(BaseError):
         changed = False
         from calibre.ebooks.oeb.polish.check.main import XML_TYPES
         check_types = XML_TYPES | OEB_DOCS
-        for name, mt in container.mime_map.iteritems():
+        for name, mt in iteritems(container.mime_map):
             if mt in check_types:
                 raw = container.raw_data(name)
                 nraw = replace_pat.sub(lambda m:html5_entities[m.group(1)], raw)
@@ -296,9 +297,9 @@ def check_xml_parsing(name, mt, raw):
             line, col = err.position
         except:
             line = col = None
-        return errors + [errcls(err.message, name, line, col)]
+        return errors + [errcls(error_message(err), name, line, col)]
     except Exception as err:
-        return errors + [errcls(err.message, name)]
+        return errors + [errcls(error_message(err), name)]
 
     if mt in OEB_DOCS:
         if root.nsmap.get(root.prefix, None) != XHTML_NS:
@@ -432,7 +433,7 @@ class BareTextInBody(BaseError):
 
 class ErrorHandler(object):
 
-    ' Replacement logger to get useful error/warning info out of cssutils during parsing '
+    ' Replacement logger to get useful error/warning info out of css_parser during parsing '
 
     def __init__(self, name):
         # may be disabled during setting of known valid items
@@ -444,7 +445,7 @@ class ErrorHandler(object):
     info = debug = setLevel = getEffectiveLevel = addHandler = removeHandler = __noop
 
     def __handle(self, level, *args):
-        msg = ' '.join(map(unicode, args))
+        msg = ' '.join(map(unicode_type, args))
         line = col = None
         for pat in pos_pats:
             m = pat.search(msg)
@@ -467,7 +468,7 @@ class ErrorHandler(object):
 
 def check_css_parsing(name, raw, line_offset=0, is_declaration=False):
     log = ErrorHandler(name)
-    parser = cssutils.CSSParser(fetcher=lambda x: (None, None), log=log)
+    parser = css_parser.CSSParser(fetcher=lambda x: (None, None), log=log)
     if is_declaration:
         parser.parseStyle(raw, validate=True)
     else:
@@ -495,7 +496,7 @@ valid_id = re.compile(r'^[a-zA-Z][a-zA-Z0-9_:.-]*$')
 def check_ids(container):
     errors = []
     mts = set(OEB_DOCS) | {guess_type('a.opf'), guess_type('a.ncx')}
-    for name, mt in container.mime_map.iteritems():
+    for name, mt in iteritems(container.mime_map):
         if mt in mts:
             root = container.parsed(name)
             seen_ids = {}
@@ -510,13 +511,13 @@ def check_ids(container):
                     seen_ids[eid] = elem.sourceline
                 if eid and valid_id.match(eid) is None:
                     errors.append(InvalidId(name, elem.sourceline, eid))
-            errors.extend(DuplicateId(name, eid, locs) for eid, locs in dups.iteritems())
+            errors.extend(DuplicateId(name, eid, locs) for eid, locs in iteritems(dups))
     return errors
 
 
 def check_markup(container):
     errors = []
-    for name, mt in container.mime_map.iteritems():
+    for name, mt in iteritems(container.mime_map):
         if mt in OEB_DOCS:
             lines = []
             root = container.parsed(name)
