@@ -7,16 +7,17 @@ __license__ = 'GPL v3'
 __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import posixpath, os, time, types
-from collections import namedtuple, defaultdict, Counter
+from collections import namedtuple, defaultdict
 from itertools import chain
 
 from calibre import prepare_string_for_xml, force_unicode
 from calibre.ebooks.oeb.base import XPath, xml2text
 from calibre.ebooks.oeb.polish.container import OEB_DOCS, OEB_STYLES, OEB_FONTS
-from calibre.ebooks.oeb.polish.spell import get_all_words
-from calibre.utils.icu import numeric_sort_key, ord_string, safe_chr
+from calibre.ebooks.oeb.polish.spell import get_all_words, count_all_chars
+from calibre.utils.icu import numeric_sort_key, safe_chr
 from calibre.utils.imghdr import identify
 from css_selectors import Select, SelectorError
+from polyglot.builtins import iteritems
 
 File = namedtuple('File', 'name dir basename size category')
 
@@ -60,10 +61,11 @@ def safe_img_data(container, name, mt):
 
 
 def files_data(container, *args):
-    for name, path in container.name_path_map.iteritems():
+    for name, path in iteritems(container.name_path_map):
         yield File(name, posixpath.dirname(name), posixpath.basename(name), safe_size(container, name),
                    get_category(name, container.mime_map.get(name, '')))
 
+
 Image = namedtuple('Image', 'name mime_type usage size basename id width height')
 
 LinkLocation = namedtuple('LinkLocation', 'name line_number text_on_line')
@@ -87,7 +89,7 @@ def safe_href_to_name(container, href, base):
 def images_data(container, *args):
     image_usage = defaultdict(set)
     link_sources = OEB_STYLES | OEB_DOCS
-    for name, mt in container.mime_map.iteritems():
+    for name, mt in iteritems(container.mime_map):
         if mt in link_sources:
             for href, line_number, offset in container.iterlinks(name):
                 target = safe_href_to_name(container, href, name)
@@ -97,7 +99,7 @@ def images_data(container, *args):
                         image_usage[target].add(LinkLocation(name, line_number, href))
 
     image_data = []
-    for name, mt in container.mime_map.iteritems():
+    for name, mt in iteritems(container.mime_map):
         if mt.startswith('image/') and container.exists(name):
             image_data.append(Image(name, mt, sort_locations(container, image_usage.get(name, set())), safe_size(container, name),
                                     posixpath.basename(name), len(image_data), *safe_img_data(container, name, mt)))
@@ -139,6 +141,7 @@ def create_anchor_map(root, pat, name):
             ans[anchor] = (LinkLocation(name, elem.sourceline, anchor), description_for_anchor(elem))
     return ans
 
+
 Anchor = namedtuple('Anchor', 'id location text')
 L = namedtuple('Link', 'location text is_external href path_ok anchor_ok anchor ok')
 
@@ -156,7 +159,7 @@ def links_data(container, *args):
     links = []
     anchor_pat = XPath('//*[@id or @name]')
     link_pat = XPath('//h:a[@href]')
-    for name, mt in container.mime_map.iteritems():
+    for name, mt in iteritems(container.mime_map):
         if mt in OEB_DOCS:
             root = container.parsed(name)
             anchor_map[name] = create_anchor_map(root, anchor_pat, name)
@@ -192,39 +195,27 @@ def links_data(container, *args):
                 link = Link(location, text, False, dest, False, False, Anchor(frag, None, None))
         yield link
 
+
 Word = namedtuple('Word', 'id word locale usage')
 
 
 def words_data(container, book_locale, *args):
     count, words = get_all_words(container, book_locale, get_word_count=True)
-    return (count, tuple(Word(i, word, locale, v) for i, ((word, locale), v) in enumerate(words.iteritems())))
+    return (count, tuple(Word(i, word, locale, v) for i, ((word, locale), v) in enumerate(iteritems(words))))
+
 
 Char = namedtuple('Char', 'id char codepoint usage count')
 
 
-def chars_data(container, *args):
-    chars = defaultdict(set)
-    counter = Counter()
-
-    def count(codepoint):
-        counter[codepoint] += 1
-
-    for name, is_linear in container.spine_names:
-        if container.mime_map.get(name) not in OEB_DOCS:
-            continue
-        raw = container.raw_data(name)
-        counts = Counter(ord_string(raw))
-        counter.update(counts)
-        for codepoint in counts:
-            chars[codepoint].add(name)
-
+def chars_data(container, book_locale, *args):
+    cc = count_all_chars(container, book_locale)
     nmap = {n:i for i, (n, l) in enumerate(container.spine_names)}
 
     def sort_key(name):
         return nmap.get(name, len(nmap)), numeric_sort_key(name)
 
-    for i, (codepoint, usage) in enumerate(chars.iteritems()):
-        yield Char(i, safe_chr(codepoint), codepoint, sorted(usage, key=sort_key), counter[codepoint])
+    for i, (codepoint, usage) in enumerate(iteritems(cc.chars)):
+        yield Char(i, safe_chr(codepoint), codepoint, sorted(usage, key=sort_key), cc.counter[codepoint])
 
 
 CSSRule = namedtuple('CSSRule', 'selector location')
@@ -262,7 +253,7 @@ def css_data(container, book_locale, result_data, *args):
     spine_names = {name for name, is_linear in container.spine_names}
     style_path, link_path = XPath('//h:style'), XPath('//h:link/@href')
 
-    for name, mt in container.mime_map.iteritems():
+    for name, mt in iteritems(container.mime_map):
         if mt in OEB_STYLES:
             importable_sheets[name] = css_rules(name, parser.parse_stylesheet(container.raw_data(name)).rules)
         elif mt in OEB_DOCS and name in spine_names:
@@ -317,7 +308,7 @@ def css_data(container, book_locale, result_data, *args):
 
     class_map = defaultdict(lambda : defaultdict(list))
 
-    for name, inline_sheets in html_sheets.iteritems():
+    for name, inline_sheets in iteritems(html_sheets):
         root = container.parsed(name)
         cmap = defaultdict(lambda : defaultdict(list))
         for elem in root.xpath('//*[@class]'):
@@ -327,21 +318,21 @@ def css_data(container, book_locale, result_data, *args):
         for sheet in chain(sheets_for_html(name, root), inline_sheets):
             for rule in rules_in_sheet(sheet):
                 rule_map[rule][name].extend(matches_for_selector(rule.selector, select, cmap, rule))
-        for cls, elem_map in cmap.iteritems():
+        for cls, elem_map in iteritems(cmap):
             class_elements = class_map[cls][name]
-            for elem, usage in elem_map.iteritems():
+            for elem, usage in iteritems(elem_map):
                 class_elements.append(
                     ClassElement(name, elem.sourceline, elem.get('class'), tag_text(elem), tuple(usage)))
 
     result_data['classes'] = ans = []
-    for cls, name_map in class_map.iteritems():
-        la = tuple(ClassFileMatch(name, tuple(class_elements), numeric_sort_key(name)) for name, class_elements in name_map.iteritems() if class_elements)
+    for cls, name_map in iteritems(class_map):
+        la = tuple(ClassFileMatch(name, tuple(class_elements), numeric_sort_key(name)) for name, class_elements in iteritems(name_map) if class_elements)
         num_of_matches = sum(sum(len(ce.matched_rules) for ce in cfm.class_elements) for cfm in la)
         ans.append(ClassEntry(cls, num_of_matches, la, numeric_sort_key(cls)))
 
     ans = []
-    for rule, loc_map in rule_map.iteritems():
-        la = tuple(CSSFileMatch(name, tuple(locations), numeric_sort_key(name)) for name, locations in loc_map.iteritems() if locations)
+    for rule, loc_map in iteritems(rule_map):
+        la = tuple(CSSFileMatch(name, tuple(locations), numeric_sort_key(name)) for name, locations in iteritems(loc_map) if locations)
         count = sum(len(fm.locations) for fm in la)
         ans.append(CSSEntry(rule, count, la, numeric_sort_key(rule.selector)))
 
