@@ -8,10 +8,9 @@ __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 import re
-from urlparse import urlparse
 from collections import Counter, OrderedDict
 from functools import partial
-from future_builtins import map
+from polyglot.builtins import iteritems, map, unicode_type
 from operator import itemgetter
 
 from lxml import etree
@@ -26,6 +25,7 @@ from calibre.ebooks.oeb.polish.opf import set_guide_item, get_book_language
 from calibre.ebooks.oeb.polish.pretty import pretty_html_tree
 from calibre.translations.dynamic import translate
 from calibre.utils.localization import get_lang, canonicalize_lang, lang_as_iso639_1
+from polyglot.urllib import urlparse
 
 ns = etree.FunctionNamespace('calibre_xpath_extensions')
 ns.prefix = 'calibre'
@@ -78,6 +78,19 @@ class TOC(object):
             for gc in child.iterdescendants():
                 yield gc
 
+    def remove_duplicates(self, only_text=True):
+        seen = set()
+        remove = []
+        for child in self:
+            key = child.title if only_text else (child.title, child.dest, (child.frag or None))
+            if key in seen:
+                remove.append(child)
+            else:
+                seen.add(key)
+                child.remove_duplicates()
+        for child in remove:
+            self.remove(child)
+
     @property
     def depth(self):
         """The maximum depth of the navigation tree rooted at this node."""
@@ -130,7 +143,7 @@ def add_from_navpoint(container, navpoint, parent, ncx_name):
         text = ''
         for txt in child_xpath(nl, 'text'):
             text += etree.tostring(txt, method='text',
-                    encoding=unicode, with_tail=False)
+                    encoding=unicode_type, with_tail=False)
     content = child_xpath(navpoint, 'content')
     if content:
         content = content[0]
@@ -155,13 +168,13 @@ def parse_ncx(container, ncx_name):
     if navmaps:
         process_ncx_node(container, navmaps[0], toc_root, ncx_name)
     toc_root.lang = toc_root.uid = None
-    for attr, val in root.attrib.iteritems():
+    for attr, val in iteritems(root.attrib):
         if attr.endswith('lang'):
-            toc_root.lang = unicode(val)
+            toc_root.lang = unicode_type(val)
             break
     for uid in root.xpath('//*[calibre:lower-case(local-name()) = "meta" and @name="dtb:uid"]/@content'):
         if uid:
-            toc_root.uid = unicode(uid)
+            toc_root.uid = unicode_type(uid)
             break
     for pl in root.xpath('//*[calibre:lower-case(local-name()) = "pagelist"]'):
         for pt in pl.xpath('descendant::*[calibre:lower-case(local-name()) = "pagetarget"]'):
@@ -178,7 +191,7 @@ def parse_ncx(container, ncx_name):
 def add_from_li(container, li, parent, nav_name):
     dest = frag = text = None
     for x in li.iterchildren(XHTML('a'), XHTML('span')):
-        text = etree.tostring(x, method='text', encoding=unicode, with_tail=False).strip() or ' '.join(x.xpath('descendant-or-self::*/@title')).strip()
+        text = etree.tostring(x, method='text', encoding=unicode_type, with_tail=False).strip() or ' '.join(x.xpath('descendant-or-self::*/@title')).strip()
         href = x.get('href')
         if href:
             dest = nav_name if href.startswith('#') else container.href_to_name(href, base=nav_name)
@@ -213,7 +226,7 @@ def parse_nav(container, nav_name):
             if ol is not None:
                 process_nav_node(container, ol, toc_root, nav_name)
                 for h in nav.iterchildren(*map(XHTML, 'h1 h2 h3 h4 h5 h6'.split())):
-                    text = etree.tostring(h, method='text', encoding=unicode, with_tail=False) or h.get('title')
+                    text = etree.tostring(h, method='text', encoding=unicode_type, with_tail=False) or h.get('title')
                     if text:
                         toc_root.toc_title = text
                         break
@@ -311,7 +324,7 @@ def get_nav_landmarks(container):
                     for a in li.iterdescendants(XHTML('a')):
                         href, rtype = a.get('href'), a.get(et)
                         if href:
-                            title = etree.tostring(a, method='text', encoding=unicode, with_tail=False).strip()
+                            title = etree.tostring(a, method='text', encoding=unicode_type, with_tail=False).strip()
                             href, frag = href.partition('#')[::2]
                             name = container.href_to_name(href, nav)
                             if container.has_name(name):
@@ -402,14 +415,14 @@ def from_xpaths(container, xpaths):
         name = container.abspath_to_name(spinepath)
         root = container.parsed(name)
         level_item_map = maps[name] = {i+1:frozenset(xp(root)) for i, xp in enumerate(xpaths)}
-        for lvl, elems in level_item_map.iteritems():
+        for lvl, elems in iteritems(level_item_map):
             if elems:
                 empty_levels.discard(lvl)
     # Remove empty levels from all level_maps
     if empty_levels:
-        for name, lmap in tuple(maps.iteritems()):
-            lmap = {lvl:items for lvl, items in lmap.iteritems() if lvl not in empty_levels}
-            lmap = sorted(lmap.iteritems(), key=itemgetter(0))
+        for name, lmap in tuple(iteritems(maps)):
+            lmap = {lvl:items for lvl, items in iteritems(lmap) if lvl not in empty_levels}
+            lmap = sorted(iteritems(lmap), key=itemgetter(0))
             lmap = {i+1:items for i, (l, items) in enumerate(lmap)}
             maps[name] = lmap
 
@@ -427,9 +440,9 @@ def from_xpaths(container, xpaths):
 
         return process_node(tocroot)
 
-    for name, level_item_map in maps.iteritems():
+    for name, level_item_map in iteritems(maps):
         root = container.parsed(name)
-        item_level_map = {e:i for i, elems in level_item_map.iteritems() for e in elems}
+        item_level_map = {e:i for i, elems in iteritems(level_item_map) for e in elems}
         item_dirtied = False
         all_ids = set(root.xpath('//*/@id'))
 
@@ -468,7 +481,10 @@ def from_links(container):
             href = a.get('href')
             if not href or not href.strip():
                 continue
-            dest = container.href_to_name(href, base=name)
+            if href.startswith('#'):
+                dest = name
+            else:
+                dest = container.href_to_name(href, base=name)
             frag = href.rpartition('#')[-1] or None
             if (dest, frag) in seen_dests:
                 continue
@@ -563,7 +579,7 @@ def create_ncx(toc, to_href, btitle, lang, uid):
         nsmap={None: NCX_NS})
     head = etree.SubElement(ncx, NCX('head'))
     etree.SubElement(head, NCX('meta'),
-        name='dtb:uid', content=unicode(uid))
+        name='dtb:uid', content=unicode_type(uid))
     etree.SubElement(head, NCX('meta'),
         name='dtb:depth', content=str(toc.depth))
     generator = ''.join(['calibre (', __version__, ')'])
@@ -655,14 +671,22 @@ def ensure_single_nav_of_type(root, ntype='toc'):
     return nav
 
 
-def commit_nav_toc(container, toc, lang=None, landmarks=None):
+def commit_nav_toc(container, toc, lang=None, landmarks=None, previous_nav=None):
     from calibre.ebooks.oeb.polish.pretty import pretty_xml_tree
     tocname = find_existing_nav_toc(container)
+    if previous_nav is not None:
+        nav_name = container.href_to_name(previous_nav[0])
+        if nav_name and container.exists(nav_name):
+            tocname = nav_name
+            container.apply_unique_properties(tocname, 'nav')
     if tocname is None:
         item = container.generate_item('nav.xhtml', id_prefix='nav')
         item.set('properties', 'nav')
         tocname = container.href_to_name(item.get('href'), base=container.opf_name)
-        root = container.parse_xhtml(P('templates/new_nav.html', data=True).decode('utf-8'))
+        if previous_nav is not None:
+            root = previous_nav[1]
+        else:
+            root = container.parse_xhtml(P('templates/new_nav.html', data=True).decode('utf-8'))
         container.replace(tocname, root)
     else:
         root = container.parsed(tocname)
