@@ -1,6 +1,7 @@
 #!/usr/bin/env python2
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 from __future__ import with_statement
+from __future__ import print_function
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -14,13 +15,13 @@ from lxml import etree
 
 from calibre import guess_type, strftime
 from calibre.constants import iswindows
-from calibre.ebooks.BeautifulSoup import BeautifulSoup
 from calibre.ebooks.oeb.base import XPath, XHTML_NS, XHTML, xml2text, urldefrag, urlnormalize
-from calibre.library.comments import comments_to_html
+from calibre.library.comments import comments_to_html, markdown
 from calibre.utils.date import is_date_undefined, as_local_time
 from calibre.utils.icu import sort_key
 from calibre.ebooks.chardet import strip_encoding_declarations
 from calibre.ebooks.metadata import fmt_sidx, rating_to_stars
+from polyglot.builtins import unicode_type
 
 JACKET_XPATH = '//h:meta[@name="calibre-content" and @content="jacket"]'
 
@@ -103,22 +104,27 @@ class Jacket(Base):
         self.log('Inserting metadata into book...')
 
         try:
-            tags = map(unicode, self.oeb.metadata.subject)
+            tags = map(unicode_type, self.oeb.metadata.subject)
         except:
             tags = []
 
         try:
-            comments = unicode(self.oeb.metadata.description[0])
+            comments = unicode_type(self.oeb.metadata.description[0])
         except:
             comments = ''
 
         try:
-            title = unicode(self.oeb.metadata.title[0])
+            title = unicode_type(self.oeb.metadata.title[0])
         except:
             title = _('Unknown')
 
+        try:
+            authors = list(map(unicode_type, self.oeb.metadata.creator))
+        except:
+            authors = [_('Unknown')]
+
         root = render_jacket(mi, self.opts.output_profile,
-                alt_title=title, alt_tags=tags,
+                alt_title=title, alt_tags=tags, alt_authors=authors,
                 alt_comments=comments, rescale_fonts=True)
         id, href = self.oeb.manifest.generate('calibre_jacket', 'jacket.xhtml')
 
@@ -137,7 +143,7 @@ class Jacket(Base):
     def remove_existing_jacket(self):
         for x in self.oeb.spine[:4]:
             if XPath(JACKET_XPATH)(x.data):
-                self.remove_images(x, limit=sys.maxint)
+                self.remove_images(x, limit=sys.maxsize)
                 self.oeb.manifest.remove(x)
                 self.log('Removed existing jacket')
                 break
@@ -170,7 +176,7 @@ def get_rating(rating, rchar, e_rchar):
     return ans
 
 
-class Series(unicode):
+class Series(unicode_type):
 
     def __new__(self, series, series_index):
         if series and series_index is not None:
@@ -180,7 +186,7 @@ class Series(unicode):
                 escape(series), escape(fmt_sidx(series_index, use_roman=False)))
         else:
             combined = roman = escape(series or u'')
-        s = unicode.__new__(self, combined)
+        s = unicode_type.__new__(self, combined)
         s.roman = roman
         s.name = escape(series or u'')
         s.number = escape(fmt_sidx(series_index or 1.0, use_roman=False))
@@ -188,19 +194,47 @@ class Series(unicode):
         return s
 
 
-class Tags(unicode):
+class Tags(unicode_type):
 
     def __new__(self, tags, output_profile):
         tags = [escape(x) for x in tags or ()]
-        t = unicode.__new__(self, ', '.join(tags))
+        t = unicode_type.__new__(self, ', '.join(tags))
         t.alphabetical = ', '.join(sorted(tags, key=sort_key))
         t.tags_list = tags
         return t
 
 
+def postprocess_jacket(root, output_profile, has_data):
+    # Post-process the generated html to strip out empty header items
+
+    def extract(tag):
+        parent = tag.getparent()
+        idx = parent.index(tag)
+        parent.remove(tag)
+        if tag.tail:
+            if idx == 0:
+                parent.text = (parent.text or '') + tag.tail
+            else:
+                if idx >= len(parent):
+                    idx = -1
+                parent[-1].tail = (parent[-1].tail or '') + tag.tail
+
+    def extract_class(cls):
+        for tag in root.xpath('//*[@class="_"]'.replace('_', cls)):
+            extract(tag)
+
+    for key in 'series rating tags'.split():
+        if not has_data[key]:
+            extract_class('cbj_' + key)
+    if not has_data['pubdate']:
+        extract_class('cbj_pubdata')
+    if output_profile.short_name != 'kindle':
+        extract_class('cbj_kindle_banner_hr')
+
+
 def render_jacket(mi, output_profile,
         alt_title=_('Unknown'), alt_tags=[], alt_comments='',
-        alt_publisher=(''), rescale_fonts=False):
+        alt_publisher='', rescale_fonts=False, alt_authors=None):
     css = P('jacket/stylesheet.css', data=True).decode('utf-8')
     template = P('jacket/template.xhtml', data=True).decode('utf-8')
 
@@ -208,7 +242,7 @@ def render_jacket(mi, output_profile,
     css = re.sub(r'/\*.*?\*/', '', css, flags=re.DOTALL)
 
     try:
-        title_str = mi.title if mi.title else alt_title
+        title_str = alt_title if mi.is_null('title') else mi.title
     except:
         title_str = _('Unknown')
     title_str = escape(title_str)
@@ -216,7 +250,7 @@ def render_jacket(mi, output_profile,
 
     series = Series(mi.series, mi.series_index)
     try:
-        publisher = mi.publisher if mi.publisher else alt_publisher
+        publisher = mi.publisher if not mi.is_null('publisher') else alt_publisher
     except:
         publisher = ''
     publisher = escape(publisher)
@@ -240,11 +274,16 @@ def render_jacket(mi, output_profile,
     if comments:
         comments = comments_to_html(comments)
 
+    orig = mi.authors
+    if mi.is_null('authors'):
+        mi.authors = list(alt_authors or (_('Unknown'),))
     try:
         author = mi.format_authors()
     except:
         author = ''
+    mi.authors = orig
     author = escape(author)
+    has_data = {}
 
     def generate_html(comments):
         args = dict(xmlns=XHTML_NS,
@@ -271,6 +310,19 @@ def render_jacket(mi, output_profile,
                     args[dkey] = Series(mi.get(key), mi.get(key + '_index'))
                 elif dt == 'rating':
                     args[dkey] = rating_to_stars(mi.get(key), m.get('display', {}).get('allow_half_stars', False))
+                elif dt == 'comments':
+                    val = val or ''
+                    display = m.get('display', {})
+                    ctype = display.get('interpret_as') or 'html'
+                    if ctype == 'long-text':
+                        val = '<pre style="white-space:pre-wrap">%s</pre>' % escape(val)
+                    elif ctype == 'short-text':
+                        val = '<span>%s</span>' % escape(val)
+                    elif ctype == 'markdown':
+                        val = markdown(val)
+                    else:
+                        val = comments_to_html(val)
+                    args[dkey] = val
                 else:
                     args[dkey] = escape(val)
                 args[dkey+'_label'] = escape(display_name)
@@ -291,33 +343,12 @@ def render_jacket(mi, output_profile,
 
         formatter = SafeFormatter()
         generated_html = formatter.format(template, **args)
+        has_data['series'] = bool(series)
+        has_data['tags'] = bool(tags)
+        has_data['rating'] = bool(rating)
+        has_data['pubdate'] = bool(pubdate)
 
-        # Post-process the generated html to strip out empty header items
-
-        soup = BeautifulSoup(generated_html)
-        if not series:
-            series_tag = soup.find(attrs={'class':'cbj_series'})
-            if series_tag is not None:
-                series_tag.extract()
-        if not rating:
-            rating_tag = soup.find(attrs={'class':'cbj_rating'})
-            if rating_tag is not None:
-                rating_tag.extract()
-        if not tags:
-            tags_tag = soup.find(attrs={'class':'cbj_tags'})
-            if tags_tag is not None:
-                tags_tag.extract()
-        if not pubdate:
-            pubdate_tag = soup.find(attrs={'class':'cbj_pubdata'})
-            if pubdate_tag is not None:
-                pubdate_tag.extract()
-        if output_profile.short_name != 'kindle':
-            hr_tag = soup.find('hr', attrs={'class':'cbj_kindle_banner_hr'})
-            if hr_tag is not None:
-                hr_tag.extract()
-
-        return strip_encoding_declarations(
-                soup.renderContents('utf-8').decode('utf-8'))
+        return strip_encoding_declarations(generated_html)
 
     from calibre.ebooks.oeb.base import RECOVER_PARSER
 
@@ -346,6 +377,7 @@ def render_jacket(mi, output_profile,
             for child in body:
                 fw.append(child)
             body.append(fw)
+    postprocess_jacket(root, output_profile, has_data)
     from calibre.ebooks.oeb.polish.pretty import pretty_html_tree
     pretty_html_tree(None, root)
     return root
