@@ -9,16 +9,17 @@ __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 import shutil, os, weakref, traceback, tempfile, time
 from threading import Thread
 from collections import OrderedDict
-from Queue import Empty
 from io import BytesIO
+from polyglot.builtins import iteritems, map, unicode_type, string_or_bytes
 
 from PyQt5.Qt import QObject, Qt, pyqtSignal
 
 from calibre import prints, as_unicode
-from calibre.constants import DEBUG
+from calibre.constants import DEBUG, iswindows, isosx, filesystem_encoding
 from calibre.customize.ui import run_plugins_on_postimport, run_plugins_on_postadd
-from calibre.db.adding import find_books_in_directory
+from calibre.db.adding import find_books_in_directory, compile_rule
 from calibre.db.utils import find_identical_books
+from calibre.ebooks.metadata import authors_to_sort_string
 from calibre.ebooks.metadata.book.base import Metadata
 from calibre.ebooks.metadata.opf2 import OPF
 from calibre.gui2 import error_dialog, warning_dialog, gprefs
@@ -28,9 +29,11 @@ from calibre.ptempfile import PersistentTemporaryDirectory
 from calibre.utils import join_with_timeout
 from calibre.utils.config import prefs
 from calibre.utils.ipc.pool import Pool, Failure
+from polyglot.queue import Empty
+
 
 def validate_source(source, parent=None):  # {{{
-    if isinstance(source, basestring):
+    if isinstance(source, string_or_bytes):
         if not os.path.exists(source):
             error_dialog(parent, _('Cannot add books'), _(
                 'The path %s does not exist') % source, show=True)
@@ -53,6 +56,7 @@ def validate_source(source, parent=None):  # {{{
     return True
 # }}}
 
+
 class Adder(QObject):
 
     do_one_signal = pyqtSignal()
@@ -61,6 +65,10 @@ class Adder(QObject):
         if not validate_source(source, parent):
             return
         QObject.__init__(self, parent)
+        self.author_map_rules = None
+        if gprefs.get('author_map_on_add_rules'):
+            from calibre.ebooks.metadata.author_mapper import compile_rules as acr
+            self.author_map_rules = acr(gprefs['author_map_on_add_rules'])
         self.single_book_per_directory = single_book_per_directory
         self.ignore_opf = False
         self.list_of_archives = list_of_archives
@@ -81,6 +89,7 @@ class Adder(QObject):
         self.report = []
         self.items = []
         self.added_book_ids = set()
+        self.merged_formats_added_to = set()
         self.merged_books = set()
         self.added_duplicate_info = set()
         self.pd.show()
@@ -119,12 +128,34 @@ class Adder(QObject):
 
     def scan(self):
 
-        def find_files(root):
-            for dirpath, dirnames, filenames in os.walk(root):
-                for files in find_books_in_directory(dirpath, self.single_book_per_directory):
-                    if self.abort_scan:
-                        return
-                    self.file_groups[len(self.file_groups)] = files
+        try:
+            compiled_rules = tuple(map(compile_rule, gprefs.get('add_filter_rules', ())))
+        except Exception:
+            compiled_rules = ()
+            import traceback
+            traceback.print_exc()
+
+        if iswindows or isosx:
+            def find_files(root):
+                for dirpath, dirnames, filenames in os.walk(root):
+                    for files in find_books_in_directory(dirpath, self.single_book_per_directory, compiled_rules=compiled_rules):
+                        if self.abort_scan:
+                            return
+                        self.file_groups[len(self.file_groups)] = files
+        else:
+            def find_files(root):
+                if isinstance(root, unicode_type):
+                    root = root.encode(filesystem_encoding)
+                for dirpath, dirnames, filenames in os.walk(root):
+                    try:
+                        dirpath = dirpath.decode(filesystem_encoding)
+                    except UnicodeDecodeError:
+                        prints('Ignoring non-decodable directory:', dirpath)
+                        continue
+                    for files in find_books_in_directory(dirpath, self.single_book_per_directory, compiled_rules=compiled_rules):
+                        if self.abort_scan:
+                            return
+                        self.file_groups[len(self.file_groups)] = files
 
         def extract(source):
             tdir = tempfile.mkdtemp(suffix='_archive', dir=self.tdir)
@@ -143,7 +174,7 @@ class Adder(QObject):
             return tdir
 
         try:
-            if isinstance(self.source, basestring):
+            if isinstance(self.source, string_or_bytes):
                 find_files(self.source)
                 self.ignore_opf = True
             else:
@@ -161,8 +192,9 @@ class Adder(QObject):
                         unreadable_files.append(path)
                 if unreadable_files:
                     if not self.file_groups:
-                        self.scan_error = _('You do not have permission to read the selected file(s).') + '\n'
-                        self.scan_error += '\n'.join(unreadable_files)
+                        m = ngettext('You do not have permission to read the selected file.',
+                                     'You do not have permission to read the selected files.', len(unreadable_files))
+                        self.scan_error = m + '\n' + '\n'.join(unreadable_files)
                     else:
                         a = self.report.append
                         for f in unreadable_files:
@@ -184,11 +216,14 @@ class Adder(QObject):
             return
         if not self.file_groups:
             error_dialog(self.pd, _('Could not add'), _(
-                'No ebook files were found in %s') % self.source, show=True)
+                'No e-book files were found in %s') % self.source, show=True)
             self.break_cycles()
             return
         self.pd.max = len(self.file_groups)
-        self.pd.title = _('Reading metadata and adding to library (%d books)...') % self.pd.max
+        self.pd.title = ngettext(
+            'Reading metadata and adding to library (one book)...',
+            'Reading metadata and adding to library ({} books)...',
+            self.pd.max).format(self.pd.max)
         self.pd.msg = ''
         self.pd.value = 0
         self.pool = Pool(name='AddBooks') if self.pool is None else self.pool
@@ -237,7 +272,7 @@ class Adder(QObject):
             except Failure as err:
                 error_dialog(self.pd, _('Cannot add books'), _(
                 'Failed to add some books, click "Show details" for more information.'),
-                det_msg=unicode(err.failure_message) + '\n' + unicode(err.details), show=True)
+                det_msg=unicode_type(err.failure_message) + '\n' + unicode_type(err.details), show=True)
                 self.pd.canceled = True
             else:
                 # All tasks completed
@@ -271,7 +306,8 @@ class Adder(QObject):
         a = self.report.append
         paths = self.file_groups[group_id]
         a(''), a('-' * 70)
-        a(_('Failed to read metadata from the file(s):'))
+        m = ngettext('Failed to read metadata from the file:', 'Failed to read metadata from the files:', len(paths))
+        a(m)
         [a('\t' + f) for f in paths]
         a(_('With error:')), a(details)
         mi = Metadata(_('Unknown'))
@@ -301,6 +337,15 @@ class Adder(QObject):
         if gprefs.get('tag_map_on_add_rules'):
             from calibre.ebooks.metadata.tag_mapper import map_tags
             mi.tags = map_tags(mi.tags, gprefs['tag_map_on_add_rules'])
+        if self.author_map_rules:
+            from calibre.ebooks.metadata.author_mapper import map_authors
+            new_authors = map_authors(mi.authors, self.author_map_rules)
+            if new_authors != mi.authors:
+                mi.authors = new_authors
+                if self.db is None:
+                    mi.author_sort = authors_to_sort_string(mi.authors)
+                else:
+                    mi.author_sort = self.db.author_sort_from_authors(mi.authors)
 
         self.pd.msg = mi.title
 
@@ -339,6 +384,7 @@ class Adder(QObject):
             ib_fmts = {fmt.upper() for fmt in self.db.formats(identical_book_id)}
             seen_fmts |= ib_fmts
             self.add_formats(identical_book_id, paths, mi, replace=replace)
+            self.merged_formats_added_to.add(identical_book_id)
         if gprefs['automerge'] == 'new record':
             incoming_fmts = {path.rpartition(os.extsep)[-1].upper() for path in paths}
             if incoming_fmts.intersection(seen_fmts):
@@ -391,7 +437,7 @@ class Adder(QObject):
     def add_formats(self, book_id, paths, mi, replace=True, is_an_add=False):
         fmap = {p.rpartition(os.path.extsep)[-1].lower():p for p in paths}
         fmt_map = {}
-        for fmt, path in fmap.iteritems():
+        for fmt, path in iteritems(fmap):
             # The onimport plugins have already been run by the read metadata
             # worker
             if self.ignore_opf and fmt.lower() == 'opf':
@@ -444,9 +490,9 @@ class Adder(QObject):
                 'Failed to add any books, click "Show details" for more information')
             d(self.pd, _('Errors while adding'), msg, det_msg='\n'.join(self.report), show=True)
 
-        if gprefs['manual_add_auto_convert'] and self.added_book_ids and self.parent() is not None:
-            self.parent().iactions['Convert Books'].auto_convert_auto_add(
-                self.added_book_ids)
+        potentially_convertible = self.added_book_ids | self.merged_formats_added_to
+        if gprefs['manual_add_auto_convert'] and potentially_convertible and self.parent() is not None:
+            self.parent().iactions['Convert Books'].auto_convert_auto_add(potentially_convertible)
 
         try:
             if callable(self.callback):
