@@ -9,6 +9,7 @@ __docformat__ = 'restructuredtext en'
 
 from calibre.gui2.convert.epub_output_ui import Ui_Form
 from calibre.gui2.convert import Widget
+from calibre.ebooks.conversion.config import OPTIONS
 
 
 class PluginWidget(Widget, Ui_Form):
@@ -19,13 +20,10 @@ class PluginWidget(Widget, Ui_Form):
     ICON = I('mimetypes/epub.png')
 
     def __init__(self, parent, get_option, get_help, db=None, book_id=None):
-        Widget.__init__(self, parent,
-                ['dont_split_on_page_breaks', 'flow_size',
-                    'no_default_epub_cover', 'no_svg_cover',
-                 'epub_inline_toc', 'epub_toc_at_end', 'toc_title',
-                    'preserve_cover_aspect_ratio', 'epub_flatten']
-                )
+        Widget.__init__(self, parent, OPTIONS['output']['epub'])
         for i in range(2):
             self.opt_no_svg_cover.toggle()
+        ev = get_option('epub_version')
+        self.opt_epub_version.addItems(list(ev.option.choices))
         self.db, self.book_id = db, book_id
         self.initialize_options(get_option, get_help, db, book_id)
