@@ -5,13 +5,14 @@ __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 
 
-import StringIO, traceback, sys, gc, weakref
+import sys, gc, weakref
 
 from PyQt5.Qt import (QMainWindow, QTimer, QAction, QMenu, QMenuBar, QIcon,
                       QObject)
 from calibre.utils.config import OptionParser
 from calibre.gui2 import error_dialog
-from calibre import prints
+from calibre import prints, force_unicode, as_unicode
+from polyglot.io import PolyglotBytesIO
 
 
 def option_parser(usage='''\
@@ -51,24 +52,24 @@ class GarbageCollector(QObject):
         # return self.debug_cycles()
         l0, l1, l2 = gc.get_count()
         if self.debug:
-            print ('gc_check called:', l0, l1, l2)
+            print('gc_check called:', l0, l1, l2)
         if l0 > self.threshold[0]:
             num = gc.collect(0)
             if self.debug:
-                print ('collecting gen 0, found:', num, 'unreachable')
+                print('collecting gen 0, found:', num, 'unreachable')
             if l1 > self.threshold[1]:
                 num = gc.collect(1)
                 if self.debug:
-                    print ('collecting gen 1, found:', num, 'unreachable')
+                    print('collecting gen 1, found:', num, 'unreachable')
                 if l2 > self.threshold[2]:
                     num = gc.collect(2)
                     if self.debug:
-                        print ('collecting gen 2, found:', num, 'unreachable')
+                        print('collecting gen 2, found:', num, 'unreachable')
 
     def debug_cycles(self):
         gc.collect()
         for obj in gc.garbage:
-            print (obj, repr(obj), type(obj))
+            print(obj, repr(obj), type(obj))
 
 
 class ExceptionHandler(object):
@@ -129,22 +130,24 @@ class MainWindow(QMainWindow):
     def set_exception_handler(self):
         sys.excepthook = ExceptionHandler(self)
 
-    def unhandled_exception(self, type, value, tb):
-        if type is KeyboardInterrupt:
+    def unhandled_exception(self, exc_type, value, tb):
+        if exc_type is KeyboardInterrupt:
             return
+        import traceback
         try:
-            sio = StringIO.StringIO()
+            sio = PolyglotBytesIO(errors='replace')
             try:
                 from calibre.debug import print_basic_debug_info
                 print_basic_debug_info(out=sio)
             except:
                 pass
-            traceback.print_exception(type, value, tb, file=sio)
+            traceback.print_exception(exc_type, value, tb, file=sio)
             if getattr(value, 'locking_debug_msg', None):
                 prints(value.locking_debug_msg, file=sio)
             fe = sio.getvalue()
             prints(fe, file=sys.stderr)
-            msg = '<b>%s</b>:'%type.__name__ + unicode(str(value), 'utf8', 'replace')
+            fe = force_unicode(fe)
+            msg = '<b>%s</b>:'%exc_type.__name__ + as_unicode(value)
             error_dialog(self, _('Unhandled exception'), msg, det_msg=fe,
                     show=True)
         except BaseException:
