@@ -12,6 +12,7 @@ from calibre.gui2.store.opensearch_store import OpenSearchOPDSStore
 from calibre.gui2.store.search_result import SearchResult
 from calibre.utils.filenames import ascii_text
 
+
 class EbooksGratuitsStore(BasicStoreConfig, OpenSearchOPDSStore):
 
     open_search_url = 'http://www.ebooksgratuits.com/opds/opensearch.xml'
@@ -21,10 +22,9 @@ class EbooksGratuitsStore(BasicStoreConfig, OpenSearchOPDSStore):
         return ascii_text(s)
 
     def search(self, query, max_results=10, timeout=60):
-        query = self.strip_accents(unicode(query))
+        query = self.strip_accents(type(u'')(query))
         for s in OpenSearchOPDSStore.search(self, query, max_results, timeout):
             if s.downloads:
                 s.drm = SearchResult.DRM_UNLOCKED
                 s.price = '$0.00'
                 yield s
-
