@@ -23,6 +23,7 @@ from calibre.ebooks.metadata import title_sort
 from calibre.gui2.dialogs.tag_categories import TagCategories
 from calibre.gui2.dialogs.tag_list_editor import TagListEditor
 from calibre.gui2.dialogs.edit_authors_dialog import EditAuthorsDialog
+from polyglot.builtins import unicode_type
 
 
 class TagBrowserMixin(object):  # {{{
@@ -121,7 +122,7 @@ class TagBrowserMixin(object):  # {{{
             if new_cat not in user_cats:
                 break
             i += 1
-            n = new_name + unicode(i)
+            n = new_name + unicode_type(i)
         # Add the new category
         user_cats[new_cat] = []
         db.new_api.set_pref('user_categories', user_cats)
@@ -267,7 +268,7 @@ class TagBrowserMixin(object):  # {{{
                     m.delete_item_from_all_user_categories(orig_name[item], category)
                 for old_id in to_rename:
                     m.rename_item_in_all_user_categories(orig_name[old_id],
-                                            category, unicode(to_rename[old_id]))
+                                            category, unicode_type(to_rename[old_id]))
 
                 db.new_api.remove_items(category, to_delete)
                 db.new_api.rename_items(category, to_rename, change_index=False)
@@ -591,7 +592,7 @@ class TagBrowserWidget(QFrame):  # {{{
 
     @property
     def find_text(self):
-        return unicode(self.item_search.currentText()).strip()
+        return unicode_type(self.item_search.currentText()).strip()
 
     def find(self):
         model = self.tags_view.model()
