@@ -12,7 +12,7 @@ from math import ceil
 from functools import partial
 from collections import namedtuple, OrderedDict
 from difflib import SequenceMatcher
-from future_builtins import zip
+from polyglot.builtins import iteritems, unicode_type, zip, range, as_bytes
 
 import regex
 from PyQt5.Qt import (
@@ -32,6 +32,7 @@ from calibre.gui2.tweak_book.diff.highlight import get_highlighter
 
 Change = namedtuple('Change', 'ltop lbot rtop rbot kind')
 
+
 class BusyCursor(object):
 
     def __enter__(self):
@@ -40,6 +41,7 @@ class BusyCursor(object):
     def __exit__(self, *args):
         QApplication.restoreOverrideCursor()
 
+
 def beautify_text(raw, syntax):
     from lxml import etree
     from calibre.ebooks.oeb.polish.parsing import parse
@@ -51,9 +53,9 @@ def beautify_text(raw, syntax):
     elif syntax == 'css':
         import logging
         from calibre.ebooks.oeb.base import serialize, _css_logger
-        from calibre.ebooks.oeb.polish.utils import setup_cssutils_serialization
-        from cssutils import CSSParser, log
-        setup_cssutils_serialization(tprefs['editor_tab_stop_width'])
+        from calibre.ebooks.oeb.polish.utils import setup_css_parser_serialization
+        from css_parser import CSSParser, log
+        setup_css_parser_serialization(tprefs['editor_tab_stop_width'])
         log.setLevel(logging.WARN)
         log.raiseExceptions = False
         parser = CSSParser(loglevel=logging.WARNING,
@@ -64,7 +66,7 @@ def beautify_text(raw, syntax):
     else:
         root = parse(raw, line_numbers=False)
         pretty_html_tree(None, root)
-    return etree.tostring(root, encoding=unicode)
+    return etree.tostring(root, encoding=unicode_type)
 
 
 class LineNumberMap(dict):  # {{{
@@ -77,7 +79,7 @@ class LineNumberMap(dict):  # {{{
         return self
 
     def __setitem__(self, k, v):
-        v = unicode(v)
+        v = unicode_type(v)
         dict.__setitem__(self, k, v)
         self.max_width = max(self.max_width, len(v))
 
@@ -86,6 +88,7 @@ class LineNumberMap(dict):  # {{{
         self.max_width = 1
 # }}}
 
+
 class TextBrowser(PlainTextEdit):  # {{{
 
     resized = pyqtSignal()
@@ -157,13 +160,13 @@ class TextBrowser(PlainTextEdit):  # {{{
 
     def calculate_metrics(self):
         w = self.fontMetrics()
-        self.number_width = max(map(lambda x:w.width(str(x)), xrange(10)))
+        self.number_width = max(map(lambda x:w.width(unicode_type(x)), range(10)))
         self.space_width = w.width(' ')
 
     def show_context_menu(self, pos):
         m = QMenu(self)
         a = m.addAction
-        i = unicode(self.textCursor().selectedText()).rstrip('\0')
+        i = unicode_type(self.textCursor().selectedText()).rstrip('\0')
         if i:
             a(QIcon(I('edit-copy.png')), _('Copy to clipboard'), self.copy).setShortcut(QKeySequence.Copy)
 
@@ -212,7 +215,7 @@ class TextBrowser(PlainTextEdit):  # {{{
         headers = dict(self.headers)
         if lnum in headers:
             cpos = self.search_header_pos
-        lines = unicode(self.toPlainText()).splitlines()
+        lines = unicode_type(self.toPlainText()).splitlines()
         for hn, text in self.headers:
             lines[hn] = text
         prefix, postfix = lines[lnum][:cpos], lines[lnum][cpos:]
@@ -303,7 +306,7 @@ class TextBrowser(PlainTextEdit):  # {{{
         while block.isValid() and top <= ev.rect().bottom():
             r = ev.rect()
             if block.isVisible() and bottom >= r.top():
-                text = unicode(self.line_number_map.get(num, ''))
+                text = unicode_type(self.line_number_map.get(num, ''))
                 is_start = text != '-' and num in change_starts
                 if is_start:
                     painter.save()
@@ -378,7 +381,7 @@ class TextBrowser(PlainTextEdit):  # {{{
         PlainTextEdit.paintEvent(self, event)
         painter = QPainter(self.viewport())
         painter.setClipRect(event.rect())
-        for top, bottom, kind in sorted(lines, key=lambda (t, b, k):{'replace':0}.get(k, 1)):
+        for top, bottom, kind in sorted(lines, key=lambda t_b_k:{'replace':0}.get(t_b_k[2], 1)):
             painter.setPen(QPen(self.diff_foregrounds[kind], 1))
             painter.drawLine(0, top, w, top)
             painter.drawLine(0, bottom - 1, w, bottom - 1)
@@ -391,6 +394,7 @@ class TextBrowser(PlainTextEdit):  # {{{
 
 # }}}
 
+
 class DiffSplitHandle(QSplitterHandle):  # {{{
 
     WIDTH = 30  # px
@@ -497,6 +501,7 @@ class DiffSplitHandle(QSplitterHandle):  # {{{
             return QSplitterHandle.wheelEvent(self, ev)
 # }}}
 
+
 class DiffSplit(QSplitter):  # {{{
 
     def __init__(self, parent=None, show_open_in_editor=False):
@@ -584,7 +589,7 @@ class DiffSplit(QSplitter):  # {{{
     def add_image_diff(self, left_data, right_data):
         def load(data):
             p = QPixmap()
-            p.loadFromData(bytes(data))
+            p.loadFromData(as_bytes(data))
             try:
                 dpr = self.devicePixelRatioF()
             except AttributeError:
@@ -607,7 +612,7 @@ class DiffSplit(QSplitter):  # {{{
             if size > 0:
                 c.beginEditBlock()
                 c.insertText(_('Size: {0} Resolution: {1}x{2}').format(human_readable(size), img.width(), img.height()))
-                for i in xrange(lines + 1):
+                for i in range(lines + 1):
                     c.insertBlock()
             change.extend((start, c.block().blockNumber()))
             c.insertBlock()
@@ -635,23 +640,24 @@ class DiffSplit(QSplitter):  # {{{
                 c.beginEditBlock()
                 c.movePosition(c.StartOfBlock)
                 if delta > 0:
-                    for _ in xrange(delta):
+                    for _ in range(delta):
                         c.insertBlock()
                 else:
                     c.movePosition(c.NextBlock, c.KeepAnchor, -delta)
                     c.removeSelectedText()
                 c.endEditBlock()
                 v.images[top] = (img, w, lines)
+
                 def mapnum(x):
                     return x if x <= top else x + delta
                 lnm = LineNumberMap()
                 lnm.max_width = v.line_number_map.max_width
-                for x, val in v.line_number_map.iteritems():
+                for x, val in iteritems(v.line_number_map):
                     dict.__setitem__(lnm, mapnum(x), val)
                 v.line_number_map = lnm
                 v.changes = [(mapnum(t), mapnum(b), k) for t, b, k in v.changes]
                 v.headers = [(mapnum(x), name) for x, name in v.headers]
-                v.images = OrderedDict((mapnum(x), v) for x, v in v.images.iteritems())
+                v.images = OrderedDict((mapnum(x), v) for x, v in iteritems(v.images))
             v.viewport().update()
 
     def get_lines_for_image(self, img, view):
@@ -741,7 +747,7 @@ class DiffSplit(QSplitter):  # {{{
     def do_insert(self, cursor, highlighter, line_number_map, lo, hi):
         start_block = cursor.block()
         highlighter.copy_lines(lo, hi, cursor)
-        for num, i in enumerate(xrange(start_block.blockNumber(), cursor.blockNumber())):
+        for num, i in enumerate(range(start_block.blockNumber(), cursor.blockNumber())):
             line_number_map[i] = lo + num + 1
         return start_block.blockNumber(), cursor.block().blockNumber()
 
@@ -800,10 +806,10 @@ class DiffSplit(QSplitter):  # {{{
         # search for the pair that matches best without being identical
         # (identical lines must be junk lines, & we don't want to synch up
         # on junk -- unless we have to)
-        for j in xrange(blo, bhi):
+        for j in range(blo, bhi):
             bj = b[j]
             cruncher.set_seq2(bj)
-            for i in xrange(alo, ahi):
+            for i in range(alo, ahi):
                 ai = a[i]
                 if ai == bj:
                     if eqi is None:
@@ -895,6 +901,7 @@ class DiffSplit(QSplitter):  # {{{
 
 # }}}
 
+
 class DiffView(QWidget):  # {{{
 
     SYNC_POSITION = 0.4
@@ -918,16 +925,18 @@ class DiffView(QWidget):  # {{{
         self.resize_timer = QTimer(self)
         self.resize_timer.setSingleShot(True)
         self.resize_timer.timeout.connect(self.resize_debounced)
-        for i, bar in enumerate((self.scrollbar, self.view.left.verticalScrollBar(), self.view.right.verticalScrollBar())):
+        for bar in (self.scrollbar, self.view.left.verticalScrollBar(), self.view.right.verticalScrollBar()):
             self.bars.append(bar)
-            bar.valueChanged[int].connect(partial(self.scrolled, i))
+            bar.scroll_idx = len(self.bars) - 1
+            connect_lambda(bar.valueChanged[int], self, lambda self: self.scrolled(self.sender().scroll_idx))
         self.view.left.resized.connect(self.resized)
-        for i, v in enumerate((self.view.left, self.view.right, self.view.handle(1))):
+        for v in (self.view.left, self.view.right, self.view.handle(1)):
             v.wheel_event.connect(self.scrollbar.wheelEvent)
-            if i < 2:
+            if v is self.view.left or v is self.view.right:
                 v.next_change.connect(self.next_change)
                 v.line_activated.connect(self.line_activated)
-                v.scrolled.connect(partial(self.scrolled, i + 1))
+                connect_lambda(v.scrolled, self,
+                        lambda self: self.scrolled(1 if self.sender() is self.view.left else 2))
 
     def next_change(self, delta):
         assert delta in (1, -1)
@@ -1086,4 +1095,3 @@ class DiffView(QWidget):  # {{{
             return True
         return False
 # }}}
-
