@@ -11,13 +11,14 @@ import json
 from PyQt5.Qt import (
     QWidget, QTimer, QStackedLayout, QLabel, QScrollArea, QVBoxLayout,
     QPainter, Qt, QPalette, QRect, QSize, QSizePolicy, pyqtSignal,
-    QColor, QMenu, QApplication, QIcon)
+    QColor, QMenu, QApplication, QIcon, QUrl)
 
-from calibre.constants import iswindows
-from calibre.gui2.tweak_book import editors, actions, current_container, tprefs
+from calibre.constants import FAKE_HOST, FAKE_PROTOCOL
+from calibre.gui2.tweak_book import editors, actions, tprefs
 from calibre.gui2.tweak_book.editor.themes import get_theme, theme_color
 from calibre.gui2.tweak_book.editor.text import default_font_family
 from css_selectors import parse, SelectorError
+from polyglot.builtins import unicode_type
 
 
 class Heading(QWidget):  # {{{
@@ -474,7 +475,7 @@ class LiveCSS(QWidget):
     def read_data(self, sourceline, tags):
         mf = self.preview.view.page().mainFrame()
         tags = [x.lower() for x in tags]
-        result = unicode(mf.evaluateJavaScript(
+        result = unicode_type(mf.evaluateJavaScript(
             'window.calibre_preview_integration.live_css(%s, %s)' % (
                 json.dumps(sourceline), json.dumps(tags))) or '')
         try:
@@ -518,12 +519,11 @@ class LiveCSS(QWidget):
         rule['properties'] = properties
 
         href = rule['href']
-        if hasattr(href, 'startswith') and href.startswith('file://'):
-            href = href[len('file://'):]
-            if iswindows and href.startswith('/'):
-                href = href[1:]
-            if href:
-                rule['href'] = current_container().abspath_to_name(href, root=self.preview.current_root)
+        if hasattr(href, 'startswith') and href.startswith('%s://%s' % (FAKE_PROTOCOL, FAKE_HOST)):
+            qurl = QUrl(href)
+            name = qurl.path()[1:]
+            if name:
+                rule['href'] = name
 
     @property
     def current_name(self):
@@ -568,4 +568,3 @@ class LiveCSS(QWidget):
             editor.goto_css_rule(data['rule_address'])
         elif data['type'] == 'elem':
             editor.goto_css_rule(data['rule_address'], sourceline_address=data['sourceline_address'])
-
