@@ -8,30 +8,29 @@ __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import time, textwrap, json
 from bisect import bisect_right
-from base64 import b64encode
-from future_builtins import map
+from polyglot.builtins import map, unicode_type, filter
 from threading import Thread
-from Queue import Queue, Empty
 from functools import partial
-from urlparse import urlparse
 
 from PyQt5.Qt import (
     QWidget, QVBoxLayout, QApplication, QSize, QNetworkAccessManager, QMenu, QIcon,
-    QNetworkReply, QTimer, QNetworkRequest, QUrl, Qt, QNetworkDiskCache, QToolBar,
+    QNetworkReply, QTimer, QNetworkRequest, QUrl, Qt, QToolBar,
     pyqtSlot, pyqtSignal)
 from PyQt5.QtWebKitWidgets import QWebView, QWebInspector, QWebPage
 
 from calibre import prints
-from calibre.constants import iswindows
+from calibre.constants import FAKE_PROTOCOL, FAKE_HOST
 from calibre.ebooks.oeb.polish.parsing import parse
 from calibre.ebooks.oeb.base import serialize, OEB_DOCS
-from calibre.ptempfile import PersistentTemporaryDirectory
-from calibre.gui2 import error_dialog, open_url, NO_URL_FORMATTING
+from calibre.gui2 import error_dialog, open_url, NO_URL_FORMATTING, secure_web_page
 from calibre.gui2.tweak_book import current_container, editors, tprefs, actions, TOP
 from calibre.gui2.viewer.documentview import apply_settings
 from calibre.gui2.viewer.config import config
 from calibre.gui2.widgets2 import HistoryLineEdit2
 from calibre.utils.ipc.simple_worker import offload_worker
+from polyglot.urllib import urlparse
+from polyglot.queue import Queue, Empty
+from polyglot.binary import as_base64_unicode
 
 shutdown = object()
 
@@ -47,7 +46,10 @@ def get_data(name):
 
 def parse_html(raw):
     root = parse(raw, decoder=lambda x:x.decode('utf-8'), line_numbers=True, linenumber_attribute='data-lnum')
-    return serialize(root, 'text/html').encode('utf-8')
+    ans = serialize(root, 'text/html')
+    if not isinstance(ans, bytes):
+        ans = ans.encode('utf-8')
+    return ans
 
 
 class ParseItem(object):
@@ -169,6 +171,12 @@ class NetworkReply(QNetworkReply):
                 data = data.encode('utf-8')
                 mime_type += '; charset=utf-8'
             self.__data = data
+            mime_type = {
+                # Prevent warning in console about mimetype of fonts
+                'application/vnd.ms-opentype':'application/x-font-ttf',
+                'application/x-font-truetype':'application/x-font-ttf',
+                'application/font-sfnt': 'application/x-font-ttf',
+            }.get(mime_type, mime_type)
             self.setHeader(QNetworkRequest.ContentTypeHeader, mime_type)
             self.setHeader(QNetworkRequest.ContentLengthHeader, len(self.__data))
             QTimer.singleShot(0, self.finalize_reply)
@@ -215,30 +223,11 @@ class NetworkReply(QNetworkReply):
 
 class NetworkAccessManager(QNetworkAccessManager):
 
-    OPERATION_NAMES = {getattr(QNetworkAccessManager, '%sOperation'%x) :
-            x.upper() for x in ('Head', 'Get', 'Put', 'Post', 'Delete',
-                'Custom')
-    }
-
-    def __init__(self, *args):
-        QNetworkAccessManager.__init__(self, *args)
-        self.current_root = None
-        self.cache = QNetworkDiskCache(self)
-        self.setCache(self.cache)
-        self.cache.setCacheDirectory(PersistentTemporaryDirectory(prefix='disk_cache_'))
-        self.cache.setMaximumCacheSize(0)
-
     def createRequest(self, operation, request, data):
-        url = unicode(request.url().toString(NO_URL_FORMATTING))
-        if operation == self.GetOperation and url.startswith('file://'):
-            path = url[7:]
-            if iswindows and path.startswith('/'):
-                path = path[1:]
+        qurl = request.url()
+        if operation == self.GetOperation and qurl.host() == FAKE_HOST:
+            name = qurl.path()[1:]
             c = current_container()
-            try:
-                name = c.abspath_to_name(path, root=self.current_root)
-            except ValueError:  # Happens on windows with absolute paths on different drives
-                name = None
             if c.has_name(name):
                 try:
                     return NetworkReply(self, request, c.mime_map.get(name, 'application/octet-stream'), name)
@@ -276,18 +265,14 @@ class WebPage(QWebPage):
         settings = self.settings()
         apply_settings(settings, config().parse())
         settings.setMaximumPagesInCache(0)
-        settings.setAttribute(settings.JavaEnabled, False)
-        settings.setAttribute(settings.PluginsEnabled, False)
+        secure_web_page(settings)
         settings.setAttribute(settings.PrivateBrowsingEnabled, True)
-        settings.setAttribute(settings.JavascriptCanOpenWindows, False)
-        settings.setAttribute(settings.JavascriptCanAccessClipboard, False)
-        settings.setAttribute(settings.LocalContentCanAccessFileUrls, False)  # ensure javascript cannot read from local files
         settings.setAttribute(settings.LinksIncludedInFocusChain, False)
         settings.setAttribute(settings.DeveloperExtrasEnabled, True)
         settings.setDefaultTextEncoding('utf-8')
         data = 'data:text/css;charset=utf-8;base64,'
         css = '[data-in-split-mode="1"] [data-is-block="1"]:hover { cursor: pointer !important; border-top: solid 5px green !important }'
-        data += b64encode(css.encode('utf-8'))
+        data += as_base64_unicode(css)
         settings.setUserStyleSheetUrl(QUrl(data))
 
         self.setNetworkAccessManager(NetworkAccessManager(self))
@@ -295,17 +280,8 @@ class WebPage(QWebPage):
         self.mainFrame().javaScriptWindowObjectCleared.connect(self.init_javascript)
         self.init_javascript()
 
-    @dynamic_property
-    def current_root(self):
-        def fget(self):
-            return self.networkAccessManager().current_root
-
-        def fset(self, val):
-            self.networkAccessManager().current_root = val
-        return property(fget=fget, fset=fset)
-
     def javaScriptConsoleMessage(self, msg, lineno, source_id):
-        prints('preview js:%s:%s:'%(unicode(source_id), lineno), unicode(msg))
+        prints('preview js:%s:%s:'%(unicode_type(source_id), lineno), unicode_type(msg))
 
     def init_javascript(self):
         if not hasattr(self, 'js'):
@@ -313,6 +289,8 @@ class WebPage(QWebPage):
             self.js = compiled_coffeescript('ebooks.oeb.display.utils', dynamic=False)
             self.js += P('csscolorparser.js', data=True, allow_user_override=False)
             self.js += compiled_coffeescript('ebooks.oeb.polish.preview', dynamic=False)
+            if isinstance(self.js, bytes):
+                self.js = self.js.decode('utf-8')
         self._line_numbers = None
         mf = self.mainFrame()
         mf.addToJavaScriptWindowObject("py_bridge", self)
@@ -321,7 +299,7 @@ class WebPage(QWebPage):
     @pyqtSlot(str, str, str)
     def request_sync(self, tag_name, href, sourceline_address):
         try:
-            self.sync_requested.emit(unicode(tag_name), unicode(href), json.loads(unicode(sourceline_address)))
+            self.sync_requested.emit(unicode_type(tag_name), unicode_type(href), json.loads(unicode_type(sourceline_address)))
         except (TypeError, ValueError, OverflowError, AttributeError):
             pass
 
@@ -332,7 +310,7 @@ class WebPage(QWebPage):
     @pyqtSlot(str, str)
     def request_split(self, loc, totals):
         actions['split-in-preview'].setChecked(False)
-        loc, totals = json.loads(unicode(loc)), json.loads(unicode(totals))
+        loc, totals = json.loads(unicode_type(loc)), json.loads(unicode_type(totals))
         if not loc or not totals:
             return error_dialog(self.view(), _('Invalid location'),
                                 _('Cannot split on the body tag'), show=True)
@@ -348,7 +326,7 @@ class WebPage(QWebPage):
                     ans = None
                 return ans
             val = self.mainFrame().evaluateJavaScript('window.calibre_preview_integration.line_numbers()')
-            self._line_numbers = sorted(uniq(filter(lambda x:x is not None, map(atoi, val))))
+            self._line_numbers = sorted(uniq(list(filter(lambda x:x is not None, map(atoi, val)))))
         return self._line_numbers
 
     def go_to_line(self, lnum):
@@ -413,14 +391,9 @@ class WebView(QWebView):
             The preview will update automatically as you make changes.
 
             <p style="font-size:x-small; color: gray">Note that this is a quick preview
-            only, it is not intended to simulate an actual ebook reader. Some
-            aspects of your ebook will not work, such as page breaks and page margins.
+            only, it is not intended to simulate an actual e-book reader. Some
+            aspects of your e-book will not work, such as page breaks and page margins.
             '''))
-        self.page().current_root = None
-
-    def setUrl(self, qurl):
-        self.page().current_root = current_container().root
-        return QWebView.setUrl(self, qurl)
 
     def inspect(self):
         self.inspector.parent().show()
@@ -432,7 +405,7 @@ class WebView(QWebView):
         p = self.page()
         mf = p.mainFrame()
         r = mf.hitTestContent(ev.pos())
-        url = unicode(r.linkUrl().toString(NO_URL_FORMATTING)).strip()
+        url = unicode_type(r.linkUrl().toString(NO_URL_FORMATTING)).strip()
         ca = self.pageAction(QWebPage.Copy)
         if ca.isEnabled():
             menu.addAction(ca)
@@ -506,19 +479,25 @@ class Preview(QWidget):
         self.search = HistoryLineEdit2(self)
         self.search.initialize('tweak_book_preview_search')
         self.search.setPlaceholderText(_('Search in preview'))
-        self.search.returnPressed.connect(partial(self.find, 'next'))
+        connect_lambda(self.search.returnPressed, self, lambda self: self.find('next'))
         self.bar.addSeparator()
         self.bar.addWidget(self.search)
         for d in ('next', 'prev'):
             ac = actions['find-%s-preview' % d]
-            ac.triggered.connect(partial(self.find, d))
+            ac.triggered.connect(getattr(self, 'find_' + d))
             self.bar.addAction(ac)
 
     def find(self, direction):
-        text = unicode(self.search.text())
+        text = unicode_type(self.search.text())
         self.view.findText(text, QWebPage.FindWrapsAroundDocument | (
             QWebPage.FindBackward if direction == 'prev' else QWebPage.FindFlags(0)))
 
+    def find_next(self):
+        self.find('next')
+
+    def find_prev(self):
+        self.find('prev')
+
     def request_sync(self, tagname, href, lnum):
         if self.current_name:
             c = current_container()
@@ -559,13 +538,19 @@ class Preview(QWidget):
             error_dialog(self, _('Failed to launch worker'), _(
                 'Failed to launch the worker process used for rendering the preview'), det_msg=tb, show=True)
 
+    def name_to_qurl(self, name=None):
+        name = name or self.current_name
+        qurl = QUrl()
+        qurl.setScheme(FAKE_PROTOCOL), qurl.setAuthority(FAKE_HOST), qurl.setPath('/' + name)
+        return qurl
+
     def show(self, name):
         if name != self.current_name:
             self.refresh_timer.stop()
             self.current_name = name
             self.report_worker_launch_error()
             parse_worker.add_request(name)
-            self.view.setUrl(QUrl.fromLocalFile(current_container().name_to_abspath(name)))
+            self.view.setUrl(self.name_to_qurl())
             return True
 
     def refresh(self):
@@ -576,7 +561,7 @@ class Preview(QWidget):
             self.report_worker_launch_error()
             parse_worker.add_request(self.current_name)
             # Tell webkit to reload all html and associated resources
-            current_url = QUrl.fromLocalFile(current_container().name_to_abspath(self.current_name))
+            current_url = self.name_to_qurl()
             self.refresh_starting.emit()
             if current_url != self.view.url():
                 # The container was changed
@@ -589,10 +574,6 @@ class Preview(QWidget):
         self.view.clear()
         self.current_name = None
 
-    @property
-    def current_root(self):
-        return self.view.page().current_root
-
     @property
     def is_visible(self):
         return actions['preview-dock'].isChecked()
