@@ -8,7 +8,7 @@ __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import time, textwrap, os
 from threading import Thread
-from future_builtins import map
+from polyglot.builtins import iteritems, map, range, filter
 from operator import itemgetter
 from functools import partial
 from collections import defaultdict
@@ -76,7 +76,7 @@ class ProxyModel(QSortFilterProxyModel):
         if not self._filter_text:
             return True
         sm = self.sourceModel()
-        for item in (sm.data(sm.index(row, c, parent)) or '' for c in xrange(sm.columnCount())):
+        for item in (sm.data(sm.index(row, c, parent)) or '' for c in range(sm.columnCount())):
             if item and primary_contains(self._filter_text, item):
                 return True
         return False
@@ -147,7 +147,7 @@ class FilesView(QTableView):
         if self.model().rowCount() > 0:
             num = min(5, self.model().rowCount())
             h = 1000000
-            for i in xrange(num):
+            for i in range(num):
                 self.resizeRowToContents(i)
                 h = min(h, self.rowHeight(i))
             self.verticalHeader().setDefaultSectionSize(h)
@@ -166,7 +166,7 @@ class FilesView(QTableView):
 
     @property
     def selected_locations(self):
-        return filter(None, (self.proxy.sourceModel().location(self.proxy.mapToSource(index)) for index in self.selectionModel().selectedIndexes()))
+        return list(filter(None, (self.proxy.sourceModel().location(self.proxy.mapToSource(index)) for index in self.selectionModel().selectedIndexes())))
 
     @property
     def current_location(self):
@@ -198,8 +198,8 @@ class FilesView(QTableView):
         w = csv_writer(buf)
         w.writerow(self.proxy.sourceModel().COLUMN_HEADERS)
         cols = self.proxy.columnCount()
-        for r in xrange(self.proxy.rowCount()):
-            items = [self.proxy.index(r, c).data(Qt.DisplayRole) for c in xrange(cols)]
+        for r in range(self.proxy.rowCount()):
+            items = [self.proxy.index(r, c).data(Qt.DisplayRole) for c in range(cols)]
             w.writerow(items)
         return buf.getvalue()
 
@@ -1092,7 +1092,7 @@ class CSSWidget(QWidget):
         buf = BytesIO()
         w = csv_writer(buf)
         w.writerow([_('Style Rule'), _('Number of matches')])
-        for r in xrange(self.proxy.rowCount()):
+        for r in range(self.proxy.rowCount()):
             entry = self.proxy.mapToSource(self.proxy.index(r, 0)).data(Qt.UserRole)
             w.writerow([entry.rule.selector, entry.count])
         return buf.getvalue()
@@ -1238,7 +1238,7 @@ class ClassesWidget(CSSWidget):
         buf = BytesIO()
         w = csv_writer(buf)
         w.writerow([_('Class'), _('Number of matches')])
-        for r in xrange(self.proxy.rowCount()):
+        for r in range(self.proxy.rowCount()):
             entry = self.proxy.mapToSource(self.proxy.index(r, 0)).data(Qt.UserRole)
             w.writerow([entry.cls, entry.num_of_matches])
         return buf.getvalue()
@@ -1328,22 +1328,22 @@ class ReportsWidget(QWidget):
         if current_page is not None:
             self.reports.setCurrentRow(current_page)
         self.layout().setContentsMargins(0, 0, 0, 0)
-        for i in xrange(self.stack.count()):
+        for i in range(self.stack.count()):
             self.stack.widget(i).layout().setContentsMargins(0, 0, 0, 0)
 
     def __call__(self, data):
         jump.clear()
-        for i in xrange(self.stack.count()):
+        for i in range(self.stack.count()):
             st = time.time()
             self.stack.widget(i)(data)
             if DEBUG:
                 category = self.reports.item(i).data(Qt.DisplayRole)
-                print ('Widget time for %12s: %.2fs seconds' % (category, time.time() - st))
+                print('Widget time for %12s: %.2fs seconds' % (category, time.time() - st))
 
     def save(self):
         save_state('splitter-state', bytearray(self.splitter.saveState()))
         save_state('report-page', self.reports.currentRow())
-        for i in xrange(self.stack.count()):
+        for i in range(self.stack.count()):
             self.stack.widget(i).save()
 
     def to_csv(self):
@@ -1439,8 +1439,8 @@ class Reports(Dialog):
                 ' information.'), det_msg=data, show=True)
         data, timing = data
         if DEBUG:
-            for x, t in sorted(timing.iteritems(), key=itemgetter(1)):
-                print ('Time for %6s data: %.3f seconds' % (x, t))
+            for x, t in sorted(iteritems(timing), key=itemgetter(1)):
+                print('Time for %6s data: %.3f seconds' % (x, t))
         self.reports(data)
 
     def accept(self):
