@@ -6,13 +6,17 @@ from __future__ import (unicode_literals, division, absolute_import,
 __license__ = 'GPL v3'
 __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import httplib, hashlib, zlib, string, time, os
+import hashlib, zlib, string, time, os
 from io import BytesIO
 from tempfile import NamedTemporaryFile
 
 from calibre import guess_type
+from calibre.constants import ispy3
 from calibre.srv.tests.base import BaseTest, TestServer
+from calibre.srv.utils import eintr_retry_call
 from calibre.utils.monotonic import monotonic
+from polyglot.builtins import iteritems, range
+from polyglot import http_client
 
 is_ci = os.environ.get('CI', '').lower() == 'true'
 
@@ -27,23 +31,23 @@ class TestHTTP(BaseTest):
             p = HTTPHeaderParser()
             p.push(*lines)
             self.assertTrue(p.finished)
-            self.assertSetEqual(set(p.hdict.items()), {(k.replace('_', '-').title(), v) for k, v in kwargs.iteritems()}, name + ' failed')
+            self.assertSetEqual(set(p.hdict.items()), {(k.replace('_', '-').title(), v) for k, v in iteritems(kwargs)}, name + ' failed')
 
         test('Continuation line parsing',
-             'a: one',
-             'b: two',
-             ' 2',
-             '\t3',
-             'c:three',
-             '\r\n', a='one', b='two 2 3', c='three')
+             b'a: one',
+             b'b: two',
+             b' 2',
+             b'\t3',
+             b'c:three',
+             b'\r\n', a='one', b='two 2 3', c='three')
 
         test('Non-ascii headers parsing',
-             b'a:mūs\r', '\r\n', a='mūs')
+             'a:mūs\r'.encode('utf-8'), b'\r\n', a='mūs')
 
         test('Comma-separated parsing',
-             'Accept-Encoding: one',
-             'accept-Encoding: two',
-             '\r\n', accept_encoding='one, two')
+             b'Accept-Encoding: one',
+             b'accept-Encoding: two',
+             b'\r\n', accept_encoding='one, two')
 
         def parse(*lines):
             lines = list(lines)
@@ -92,9 +96,9 @@ class TestHTTP(BaseTest):
             def test(al, q):
                 conn.request('GET', '/', headers={'Accept-Language': al})
                 r = conn.getresponse()
-                self.ae(r.status, httplib.OK)
-                q += get_translator(q)[-1].ugettext('Unknown')
-                self.ae(r.read(), q)
+                self.ae(r.status, http_client.OK)
+                q += getattr(get_translator(q)[-1], 'gettext' if ispy3 else 'ugettext')('Unknown')
+                self.ae(r.read(), q.encode('utf-8'))
 
             test('en', 'en')
             test('eng', 'en')
@@ -134,7 +138,7 @@ class TestHTTP(BaseTest):
 
         def raw_send(conn, raw):
             conn.send(raw)
-            conn._HTTPConnection__state = httplib._CS_REQ_SENT
+            conn._HTTPConnection__state = http_client._CS_REQ_SENT
             return conn.getresponse()
 
         base_timeout = 0.5 if is_ci else 0.1
@@ -142,39 +146,39 @@ class TestHTTP(BaseTest):
         with TestServer(handler, timeout=base_timeout, max_header_line_size=100./1024, max_request_body_size=100./(1024*1024)) as server:
             conn = server.connect()
             r = raw_send(conn, b'hello\n')
-            self.ae(r.status, httplib.BAD_REQUEST)
+            self.ae(r.status, http_client.BAD_REQUEST)
             self.ae(r.read(), b'HTTP requires CRLF line terminators')
 
             r = raw_send(conn, b'\r\nGET /index.html HTTP/1.1\r\n\r\n')
-            self.ae(r.status, httplib.NOT_FOUND), self.ae(r.read(), b'Requested resource not found')
+            self.ae(r.status, http_client.NOT_FOUND), self.ae(r.read(), b'Requested resource not found')
 
             r = raw_send(conn, b'\r\n\r\nGET /index.html HTTP/1.1\r\n\r\n')
-            self.ae(r.status, httplib.BAD_REQUEST)
+            self.ae(r.status, http_client.BAD_REQUEST)
             self.ae(r.read(), b'Multiple leading empty lines not allowed')
 
             r = raw_send(conn, b'hello world\r\n')
-            self.ae(r.status, httplib.BAD_REQUEST)
+            self.ae(r.status, http_client.BAD_REQUEST)
             self.ae(r.read(), b'Malformed Request-Line')
 
             r = raw_send(conn, b'x' * 200)
-            self.ae(r.status, httplib.BAD_REQUEST)
+            self.ae(r.status, http_client.BAD_REQUEST)
             self.ae(r.read(), b'')
 
             r = raw_send(conn, b'XXX /index.html HTTP/1.1\r\n\r\n')
-            self.ae(r.status, httplib.BAD_REQUEST), self.ae(r.read(), b'Unknown HTTP method')
+            self.ae(r.status, http_client.BAD_REQUEST), self.ae(r.read(), b'Unknown HTTP method')
 
             # Test 404
             conn.request('HEAD', '/moose')
             r = conn.getresponse()
-            self.ae(r.status, httplib.NOT_FOUND)
+            self.ae(r.status, http_client.NOT_FOUND)
             self.assertIsNotNone(r.getheader('Date', None))
             self.ae(r.getheader('Content-Length'), str(len(body)))
             self.ae(r.getheader('Content-Type'), 'text/plain; charset=UTF-8')
             self.ae(len(r.getheaders()), 3)
-            self.ae(r.read(), '')
+            self.ae(r.read(), b'')
             conn.request('GET', '/choose')
             r = conn.getresponse()
-            self.ae(r.status, httplib.NOT_FOUND)
+            self.ae(r.status, http_client.NOT_FOUND)
             self.ae(r.read(), b'Requested resource not found')
 
             # Test 500
@@ -184,7 +188,7 @@ class TestHTTP(BaseTest):
             conn = server.connect()
             conn.request('GET', '/test/')
             r = conn.getresponse()
-            self.ae(r.status, httplib.INTERNAL_SERVER_ERROR)
+            self.ae(r.status, http_client.INTERNAL_SERVER_ERROR)
             server.loop.log.filter_level = orig
 
             # Test 301
@@ -194,9 +198,9 @@ class TestHTTP(BaseTest):
             conn = server.connect()
             conn.request('GET', '/')
             r = conn.getresponse()
-            self.ae(r.status, httplib.MOVED_PERMANENTLY)
+            self.ae(r.status, http_client.MOVED_PERMANENTLY)
             self.ae(r.getheader('Location'), '/somewhere-else')
-            self.ae('', r.read())
+            self.ae(b'', r.read())
 
             server.change_handler(lambda data:data.path[0] + data.read().decode('ascii'))
             conn = server.connect(timeout=base_timeout * 5)
@@ -204,26 +208,26 @@ class TestHTTP(BaseTest):
             # Test simple GET
             conn.request('GET', '/test/')
             r = conn.getresponse()
-            self.ae(r.status, httplib.OK)
+            self.ae(r.status, http_client.OK)
             self.ae(r.read(), b'test')
 
             # Test TRACE
             lines = ['TRACE /xxx HTTP/1.1', 'Test: value', 'Xyz: abc, def', '', '']
             r = raw_send(conn, ('\r\n'.join(lines)).encode('ascii'))
-            self.ae(r.status, httplib.OK)
+            self.ae(r.status, http_client.OK)
             self.ae(r.read().decode('utf-8'), '\n'.join(lines[:-2]))
 
             # Test POST with simple body
             conn.request('POST', '/test', 'body')
             r = conn.getresponse()
-            self.ae(r.status, httplib.OK)
+            self.ae(r.status, http_client.OK)
             self.ae(r.read(), b'testbody')
 
             # Test POST with chunked transfer encoding
             conn.request('POST', '/test', headers={'Transfer-Encoding': 'chunked'})
             conn.send(b'4\r\nbody\r\na\r\n1234567890\r\n0\r\n\r\n')
             r = conn.getresponse()
-            self.ae(r.status, httplib.OK)
+            self.ae(r.status, http_client.OK)
             self.ae(r.read(), b'testbody1234567890')
 
             # Test various incorrect input
@@ -231,61 +235,63 @@ class TestHTTP(BaseTest):
 
             conn.request('GET', '/test' + ('a' * 200))
             r = conn.getresponse()
-            self.ae(r.status, httplib.BAD_REQUEST)
+            self.ae(r.status, http_client.BAD_REQUEST)
 
             conn = server.connect()
             conn.request('GET', '/test', ('a' * 200))
             r = conn.getresponse()
-            self.ae(r.status, httplib.REQUEST_ENTITY_TOO_LARGE)
+            self.ae(r.status, http_client.REQUEST_ENTITY_TOO_LARGE)
 
             conn = server.connect()
             conn.request('POST', '/test', headers={'Transfer-Encoding': 'chunked'})
             conn.send(b'x\r\nbody\r\n0\r\n\r\n')
             r = conn.getresponse()
-            self.ae(r.status, httplib.BAD_REQUEST)
+            self.ae(r.status, http_client.BAD_REQUEST)
             self.assertIn(b'not a valid chunk size', r.read())
 
             conn.request('POST', '/test', headers={'Transfer-Encoding': 'chunked'})
             conn.send(b'4\r\nbody\r\n200\r\n\r\n')
             r = conn.getresponse()
-            self.ae(r.status, httplib.REQUEST_ENTITY_TOO_LARGE)
+            self.ae(r.status, http_client.REQUEST_ENTITY_TOO_LARGE)
             conn.request('POST', '/test', body='a'*200)
             r = conn.getresponse()
-            self.ae(r.status, httplib.REQUEST_ENTITY_TOO_LARGE)
+            self.ae(r.status, http_client.REQUEST_ENTITY_TOO_LARGE)
 
             conn = server.connect()
             conn.request('POST', '/test', headers={'Transfer-Encoding': 'chunked'})
             conn.send(b'3\r\nbody\r\n0\r\n\r\n')
             r = conn.getresponse()
-            self.ae(r.status, httplib.BAD_REQUEST), self.ae(r.read(), b'Chunk does not have trailing CRLF')
+            self.ae(r.status, http_client.BAD_REQUEST), self.ae(r.read(), b'Chunk does not have trailing CRLF')
 
             conn = server.connect(timeout=base_timeout * 5)
             conn.request('POST', '/test', headers={'Transfer-Encoding': 'chunked'})
             conn.send(b'30\r\nbody\r\n0\r\n\r\n')
             r = conn.getresponse()
-            self.ae(r.status, httplib.REQUEST_TIMEOUT)
+            self.ae(r.status, http_client.REQUEST_TIMEOUT)
             self.assertIn(b'', r.read())
 
             server.log.filter_level = orig_level
             conn = server.connect()
             # Test pipelining
             responses = []
-            for i in xrange(10):
-                conn._HTTPConnection__state = httplib._CS_IDLE
+            for i in range(10):
+                conn._HTTPConnection__state = http_client._CS_IDLE
                 conn.request('GET', '/%d'%i)
-                responses.append(conn.response_class(conn.sock, strict=conn.strict, method=conn._method))
-            for i in xrange(10):
+                if ispy3:
+                    responses.append(conn.response_class(conn.sock, method=conn._method))
+                else:
+                    responses.append(conn.response_class(conn.sock, strict=conn.strict, method=conn._method))
+            for i in range(10):
                 r = responses[i]
                 r.begin()
                 self.ae(r.read(), ('%d' % i).encode('ascii'))
-            conn._HTTPConnection__state = httplib._CS_IDLE
+            conn._HTTPConnection__state = http_client._CS_IDLE
 
             # Test closing
             server.loop.opts.timeout = 10  # ensure socket is not closed because of timeout
             conn.request('GET', '/close', headers={'Connection':'close'})
             r = conn.getresponse()
-            self.ae(server.loop.num_active_connections, 1)
-            self.ae(r.status, 200), self.ae(r.read(), 'close')
+            self.ae(r.status, 200), self.ae(r.read(), b'close')
             server.loop.wakeup()
             num = 10
             while num and server.loop.num_active_connections != 0:
@@ -293,6 +299,14 @@ class TestHTTP(BaseTest):
                 num -= 1
             self.ae(server.loop.num_active_connections, 0)
             self.assertIsNone(conn.sock)
+
+            # Test timeout
+            server.loop.opts.timeout = 0.1
+            conn = server.connect(timeout=1)
+            conn.request('GET', '/something')
+            r = conn.getresponse()
+            self.ae(r.status, 200), self.ae(r.read(), b'something')
+            self.assertIn(b'Request Timeout', eintr_retry_call(conn.sock.recv, 500))
     # }}}
 
     def test_http_response(self):  # {{{
@@ -303,19 +317,19 @@ class TestHTTP(BaseTest):
             return conn.generate_static_output('test', lambda : ''.join(conn.path))
         with NamedTemporaryFile(suffix='test.epub') as f, open(P('localization/locales.zip'), 'rb') as lf, \
                 TestServer(handler, timeout=1, compress_min_size=0) as server:
-            fdata = string.ascii_letters * 100
+            fdata = (string.ascii_letters * 100).encode('ascii')
             f.write(fdata), f.seek(0)
 
             # Test ETag
             conn = server.connect()
             conn.request('GET', '/an_etagged_path')
             r = conn.getresponse()
-            self.ae(r.status, httplib.OK), self.ae(r.read(), b'an_etagged_path')
+            self.ae(r.status, http_client.OK), self.ae(r.read(), b'an_etagged_path')
             etag = r.getheader('ETag')
-            self.ae(etag, '"%s"' % hashlib.sha1('an_etagged_path').hexdigest())
+            self.ae(etag, '"%s"' % hashlib.sha1(b'an_etagged_path').hexdigest())
             conn.request('GET', '/an_etagged_path', headers={'If-None-Match':etag})
             r = conn.getresponse()
-            self.ae(r.status, httplib.NOT_MODIFIED)
+            self.ae(r.status, http_client.NOT_MODIFIED)
             self.ae(r.read(), b'')
 
             # Test gzip
@@ -325,7 +339,7 @@ class TestHTTP(BaseTest):
             conn.request('GET', '/an_etagged_path', headers={'Accept-Encoding':'gzip'})
             r = conn.getresponse()
             self.ae(str(len(raw)), r.getheader('Calibre-Uncompressed-Length'))
-            self.ae(r.status, httplib.OK), self.ae(zlib.decompress(r.read(), 16+zlib.MAX_WBITS), raw)
+            self.ae(r.status, http_client.OK), self.ae(zlib.decompress(r.read(), 16+zlib.MAX_WBITS), raw)
 
             # Test dynamic etagged content
             num_calls = [0]
@@ -337,13 +351,13 @@ class TestHTTP(BaseTest):
             conn = server.connect()
             conn.request('GET', '/an_etagged_path')
             r = conn.getresponse()
-            self.ae(r.status, httplib.OK), self.ae(r.read(), b'data')
+            self.ae(r.status, http_client.OK), self.ae(r.read(), b'data')
             etag = r.getheader('ETag')
-            self.ae(etag, b'"xxx"')
+            self.ae(etag, '"xxx"')
             self.ae(r.getheader('Content-Length'), '4')
             conn.request('GET', '/an_etagged_path', headers={'If-None-Match':etag})
             r = conn.getresponse()
-            self.ae(r.status, httplib.NOT_MODIFIED)
+            self.ae(r.status, http_client.NOT_MODIFIED)
             self.ae(r.read(), b'')
             self.ae(num_calls[0], 1)
 
@@ -359,11 +373,11 @@ class TestHTTP(BaseTest):
                 self.ae(r.getheader('Content-Type'), guess_type(f.name)[0])
                 self.ae(type('')(r.getheader('Accept-Ranges')), 'bytes')
                 self.ae(int(r.getheader('Content-Length')), len(fdata))
-                self.ae(r.status, httplib.OK), self.ae(r.read(), fdata)
+                self.ae(r.status, http_client.OK), self.ae(r.read(), fdata)
 
                 conn.request('GET', '/test', headers={'Range':'bytes=2-25'})
                 r = conn.getresponse()
-                self.ae(r.status, httplib.PARTIAL_CONTENT)
+                self.ae(r.status, http_client.PARTIAL_CONTENT)
                 self.ae(type('')(r.getheader('Accept-Ranges')), 'bytes')
                 self.ae(type('')(r.getheader('Content-Range')), 'bytes 2-25/%d' % len(fdata))
                 self.ae(int(r.getheader('Content-Length')), 24)
@@ -371,27 +385,27 @@ class TestHTTP(BaseTest):
 
                 conn.request('GET', '/test', headers={'Range':'bytes=100000-'})
                 r = conn.getresponse()
-                self.ae(r.status, httplib.REQUESTED_RANGE_NOT_SATISFIABLE)
+                self.ae(r.status, http_client.REQUESTED_RANGE_NOT_SATISFIABLE)
                 self.ae(type('')(r.getheader('Content-Range')), 'bytes */%d' % len(fdata))
 
                 conn.request('GET', '/test', headers={'Range':'bytes=25-50', 'If-Range':etag})
                 r = conn.getresponse()
-                self.ae(r.status, httplib.PARTIAL_CONTENT), self.ae(r.read(), fdata[25:51])
+                self.ae(r.status, http_client.PARTIAL_CONTENT), self.ae(r.read(), fdata[25:51])
                 self.ae(int(r.getheader('Content-Length')), 26)
 
                 conn.request('GET', '/test', headers={'Range':'bytes=0-1000000'})
                 r = conn.getresponse()
-                self.ae(r.status, httplib.PARTIAL_CONTENT), self.ae(r.read(), fdata)
+                self.ae(r.status, http_client.PARTIAL_CONTENT), self.ae(r.read(), fdata)
 
                 conn.request('GET', '/test', headers={'Range':'bytes=25-50', 'If-Range':'"nomatch"'})
                 r = conn.getresponse()
-                self.ae(r.status, httplib.OK), self.ae(r.read(), fdata)
+                self.ae(r.status, http_client.OK), self.ae(r.read(), fdata)
                 self.assertFalse(r.getheader('Content-Range'))
                 self.ae(int(r.getheader('Content-Length')), len(fdata))
 
                 conn.request('GET', '/test', headers={'Range':'bytes=0-25,26-50'})
                 r = conn.getresponse()
-                self.ae(r.status, httplib.PARTIAL_CONTENT)
+                self.ae(r.status, http_client.PARTIAL_CONTENT)
                 clen = int(r.getheader('Content-Length'))
                 data = r.read()
                 self.ae(clen, len(data))
@@ -406,7 +420,7 @@ class TestHTTP(BaseTest):
                 conn = server.connect(timeout=1)
                 conn.request('GET', '/test')
                 r = conn.getresponse()
-                self.ae(r.status, httplib.OK)
+                self.ae(r.status, http_client.OK)
                 rdata = r.read()
                 self.ae(len(data), len(rdata))
                 self.ae(hashlib.sha1(data).hexdigest(), hashlib.sha1(rdata).hexdigest())
@@ -418,7 +432,7 @@ class TestHTTP(BaseTest):
 
     def test_static_generation(self):  # {{{
         'Test static generation'
-        nums = list(map(str, xrange(10)))
+        nums = list(map(str, range(10)))
 
         def handler(conn):
             return conn.generate_static_output('test', nums.pop)
@@ -427,9 +441,8 @@ class TestHTTP(BaseTest):
             conn.request('GET', '/an_etagged_path')
             r = conn.getresponse()
             data = r.read()
-            for i in xrange(5):
+            for i in range(5):
                 conn.request('GET', '/an_etagged_path')
                 r = conn.getresponse()
                 self.assertEqual(data, r.read())
     # }}}
-
